/**
 * @author N. Kornweibel
 * @version 1.0
 * An observation, a part of a program, as represented in Gasgano.
 */

package org.eso.gasgano.datamodel;

import java.util.Enumeration;

import org.eso.gasgano.datamodel.gui.OBComponent;
import org.eso.gasgano.properties.GasProp;
import org.eso.gasgano.tools.SortedHashtable;

public class Observation {

    private String id = null;
    private String desc = null;
    private Program program = null;

    /**
     * The set of all OBComponents under this observation.
     */
    private SortedHashtable obComponents = null;

    /**
     * The set of all Intermediate containers  under this observation.
     */
    private SortedHashtable intermediates = null;

    /**
     * @param name The id of this Observation.
     */
    public Observation(String name) {
        id = name;
        obComponents = new SortedHashtable(true);
        intermediates = new SortedHashtable();
    }

    /**
     * @return The id of this Observation.
     */
    public String getId() {
        return id;
    }

    /**
     * @return The description of this Observation, or the id if the
     * description is null.
     */
    public String toString() {
        String rc = desc;
        if (rc == null) {
            rc = getId();
        }
        return rc;
    }

    /**
     * @param s The description to be set for this Observation.
     * @see org.eso.gasgano.ObservationDescription
     */
    public void setDescription(String s) {
        desc = s;
    }

    /** 
     * @param obc The OBCompenent to be added under this Observation.
     * An OBComponent can only be added once.
     */
    public void addObComponent(OBComponent obc) {
        if (getComponent(obc.getId()) == null) {
            obComponents.put(obc.getId(), obc);
        }
    }

    /** 
     * @param obc The OBCompenent to be removed from this Observation.
     */
    public void removeObComponent(OBComponent obc) {
        obComponents.remove(obc.getId());
    }

    /**
     * @param id The id of the OBComponent to be located.
     * @return The OBComponent located, or null if not located.
     */
    public OBComponent getComponent(String id) {
        return (OBComponent) obComponents.get(id);
    }

    /**
     * @return True if there are no OBComponent under this Observation.
     */
    public boolean isEmpty() {
        return obComponents.isEmpty() && intermediates.isEmpty();
    }

    /**
     * @return The SortedHashtable of OBComponent under this Observation.
     */
    public SortedHashtable getComponents() {
        return obComponents;
    }

    public OBComponent getComponentAt(int i) {
        return (OBComponent) obComponents.elementAt(i);
    }

    public int getIndexOfComponent(OBComponent comp) {
        return obComponents.indexOf(comp);
    }

    public int getComponentCount() {
        return obComponents.size();
    }

    public void cleanup() {
        obComponents.clear();
    }

    public void setProgram(Program program) {
        this.program = program;
    }

    public Program getProgram() {
        return program;
    }

    /**
     * Get the named intermediate object, or null if not found.
     * @param name
     * @return InterContainer
     */
    public InterContainer findIntermediate(String name) {
        return (InterContainer)intermediates.get(name);
    }
    
    /**
     * Add an intermediate. This method should not be used directly as intermediates should
     * be added to Observation objects through DFSDataModel.addIntermediate().
     * This ensures correct fireing of events to listeners. The method is public because the
     * data model is currently part of a sub package of this package (not ideal!).
     * @param name of the intermediate to add.
     * @return InterContainer the inter container (group) created, or the existing
     * one if already existing.
     */
    public InterContainer addIntermediate(String name) {
        InterContainer rc = null;
        rc = findIntermediate(name);
        if (rc == null) {
            rc = new InterContainer(name);
            intermediates.put(name, rc);
        }
        return rc;       
    }
    
    public SortedHashtable getIntermediates() {
        return intermediates;
    }

    public InterContainer getIntermediateAt(int i) {
        return (InterContainer) intermediates.elementAt(i);
    }

    public int getIndexOfIntermediate(InterContainer inter) {
        return intermediates.indexOf(inter);
    }

    public int getIntermediateCount() {
        return intermediates.size();
    }

    /**
     * Remove the given intermediate from the Observation. This method should
     * not be called directly, instead the DFSDataModel.removeIntermediate() 
     * should be called to insure correct firing of events to listeners.
     * @param inter
     */
    public void removeIntermediate(InterContainer inter) {
        intermediates.remove(inter.getId());
    }

    public SortedHashtable getDescendants() {
        SortedHashtable tmpHash = null;
        if (getIntermediateCount() == 0) {
            return getComponents();
        }
        if (getComponentCount() == 0) {
            return getIntermediates();
        }

        tmpHash = new SortedHashtable();
        for (Enumeration e = intermediates.keys(); e.hasMoreElements();) {
            Object tmpObject = e.nextElement();
            tmpHash.put(tmpObject, intermediates.get(tmpObject));
        }
        for (Enumeration e = obComponents.keys(); e.hasMoreElements();) {
            Object tmpObject = e.nextElement();
            tmpHash.put(tmpObject, obComponents.get(tmpObject));
        }
        return tmpHash;
    }
}

// end of file Observation.java.
