/**
 * @author N. Kornweibel
 * @version 1.0
 * Representation of all ASCII Text files.
 */
package org.eso.gasgano.datamodel.filesystem;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;

public class DFSTextFile extends DFSFile {

    String dataAddress = null;
    String data = null;
    
    /**
     * Constructor for a DFSTextFile object, the address being the full path to the ASCII file.
     * @param a The address of the file.
     */
    public DFSTextFile(String a) {
        dataAddress = a;
        readData();
    }

    /**
     * @return The name of this file.
     */
    public String toString() {
        return getFileName();
    }

    /**
     * Read the file data. Set the file status and last update time. 
     * The file data is released from memory after reading and the status returned.
     * @return The status of the this file after parsing.
     * @see org.eso.gasgano.DFSFile
     */
    public int readData() {
        dataRefreshed();
        int rc = readFile();
        // free the memory now that we have finished with it.
        data = null;
        return rc;
    }

    /**
     * This method will <b>not</b> release data read into memory.
     * @return The status of the this file after parsing.
     * @see org.eso.gasgano.DFSFile
     */
    public int readFile() {
        setStatus(FILE_VALID);
        File theFile = new File(dataAddress);

        FileReader dataSource = null;
        try {
            dataSource = new FileReader(dataAddress);
        } catch (FileNotFoundException ex) {
            System.out.println("File not found: "+dataAddress);
            setStatus(FILE_NOT_FOUND);
            return getStatus();
        }
        if (dataSource == null) {
            setStatus(FILE_INVALID);
            return getStatus();
        }
        StringBuffer buff = new StringBuffer();
        BufferedReader br = new BufferedReader(dataSource);
        try {
            while (true) {
                String s = br.readLine();
                if (s == null) {
                    break;
                } else {
                    buff.append(s+"\n");
                }
            }
        } catch (IOException e) {
            // error reading from the Reader.
            setStatus(FILE_INVALID);
        }
        try {
            dataSource.close();
        } catch (IOException e) {}

        data = buff.toString();
        return getStatus();
    } // end of method readFile().

    /**
     * Return the file content.
     * @return The ASCII data read in from readFile() or null.
     */
    public String getData() {
        return data;
    }

    /**
     * Release the data read in with readFile().
     */
    public void freeData() {
        data = null;
    }

    /**
     * @return The a File object for this file.
     */
    public File getPath() {
        return new File(dataAddress);
    }

    /**
     * @return the original address with which the object was created.
     * The Path object returned by getPath() will return the resolved path
     * name, with file system links resolved.
     */
    public String getAddress() {
        return dataAddress;
    }

    /**
     * Sets the data.
     * @param data The data to set
     */
    public void setData(String data) {
        this.data = data;
    }

} // end of class DFSTextFile.java

// end of file DFSTextFile.java
