/**
 * @author M. Zamparelli
 * @version 1.0
 * A class for defining a common set of methods common to all 
 * those components which have a Keyword blocks in their header.
 */

package org.eso.gasgano.datamodel.gui;

import java.io.File;
import java.util.Vector;

import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.SwingConstants;

import org.eso.gasgano.datamodel.DataModelEvent;
import org.eso.gasgano.datamodel.InterContainer;
import org.eso.gasgano.datamodel.filesystem.DFSDataModel;
import org.eso.gasgano.datamodel.filesystem.DFSFile;
import org.eso.gasgano.datamodel.filesystem.FitsFile;
import org.eso.gasgano.keyword.Keyword;
import org.eso.gasgano.keyword.RuleTable;
import org.eso.gasgano.properties.GasProp;

public abstract class KeywordContainerOBComponent extends OBComponent {

    protected FitsFile myFile = null;
    private String[] columnDisplayData;
    protected String arcFile = null;

    private static final int numberIdKeys = 3;
    private static final String fakeSignature =
        "ffffffffffffffffffffffffffffffff";
    protected static Keyword[] defaultKeywordQuery = null;

    public static long initTime = 0;
    public static int initNumber = 0;

    /**
     * @param address The path of the fits file.
     */
    public KeywordContainerOBComponent(String address) {
        super();
        init(address);
    }

    public abstract void addToDataModel(
        String progId,
        String obId,
        String telescopeId);

    /**
     * @param address The address of the fits file.
     */
    public void init(String address) {
        RuleTable rt = DFSDataModel.getDataModel().getRuleTable();
        if (defaultKeywordQuery == null) {
            setDefaultKeywordQuery(null);
        }
        // null the values in the defaultKeywordQuery
        for (int i = 0; i < defaultKeywordQuery.length; i++) {
            defaultKeywordQuery[i].setValue(null);
        }
        myFile = new FitsFile(address, rt, defaultKeywordQuery);

        // copy the keyword value back into the display keys.

        setPipeProduct(myFile.isPipelineProduct());
        setDescription();

        if (!isPipeProduct()) {
            File tmpFile = new File(address);
            arcFile =
                new String(
                    tmpFile.getParent()
                        + File.separator
                        + defaultKeywordQuery[2].getValue());
        }
    }

    public String getClassification() {
        return myFile.getClassification();
    }

    public String getInstrument() {
        if (myFile.getInstrument() != null) {
            return myFile.getInstrument();
        } else {
            // System.out.println("Empty INSTRUMENT keyword "+this);
            return null;
        }
    }

    /**
     * Determine if this OBComponent has an Intermediate (i.e. it
     * is placed in groups under the Observation). This is determined
     * from the instrument and Gasgano preferrences.
     * @return boolean true if this obcomponent will be placed in
     * a group which is placed under and observation. if false, then
     * the obcomponent will be placed directly under the observation.
     */
    public boolean hasIntermediate() {
        String instr = getInstrument();
        if ((GasProp.getProperties().hasInstrumentGrouping(instr))
            && (getIntermediateGrouping() != null)) {
            return true;
        }
        return false;
    }

    public void classify() {
        myFile.classify(DFSDataModel.getDataModel().getRuleTable());
        if (getDefaultFileDescription() != null) {
            getDefaultFileDescription().classificationChanged(this);
        }
    }

    // ********************* DefaultKeywordQuery  area  ************************

    public static void setDefaultKeywordQueryToNull() {
        for (int i = numberIdKeys; i < defaultKeywordQuery.length; i++) {
            defaultKeywordQuery[i].setValue("");
        }
    }

    /**
     * @param k The array of Keywords to be filled in when reading in a
     * Fits file header. This array may be added to.
     */
    public static void setDefaultKeywordQuery(Keyword[] k) {
        int klen = 0;
        if (k != null) {
            klen = k.length;
        }
        defaultKeywordQuery = new Keyword[numberIdKeys + klen];
        defaultKeywordQuery[0] = new Keyword(Keyword.PROG_ID, null);
        defaultKeywordQuery[1] = new Keyword(Keyword.OBS_ID, null);
        defaultKeywordQuery[2] = new Keyword(Keyword.ARCFILE, null);
        if (k != null) {
            System.arraycopy(k, 0, defaultKeywordQuery, numberIdKeys, k.length);
        }
    }
    /**
     * @param k The array of Keywords to be added to the
     * default keyword query set.
     */
    public static void addToDefaultKeywordQuery(Keyword[] k) {
        if (defaultKeywordQuery == null) {
            setDefaultKeywordQuery(k);
        } else if (k != null) {
            Keyword[] temp = defaultKeywordQuery;
            defaultKeywordQuery = new Keyword[temp.length + k.length];
            System.arraycopy(temp, 0, defaultKeywordQuery, 0, temp.length);
            System.arraycopy(k, 0, defaultKeywordQuery, temp.length, k.length);
        }
    }

    /**
     * @return The array of Keywords filled in with each FitsFile created.
     * Warning this array is a static member of this class - the
     * array has been filled with the values of the last FitsFile
     * created. While this is valid that the value are correct for
     * a non-threaded application, it is problematic for a multi-threaded
     * application.
     */
    public static Keyword[] getDefaultKeywordQuery() {
        return defaultKeywordQuery;
    }

    /**
     *	get the array of data for the TreeTable display.
     */
    public String[] getDisplayData() {
        return columnDisplayData;
    }

    /**
     * set the array of data for the TreeTable display.
     */
    public void setDisplayData(String[] cols) {
        columnDisplayData = cols;
    }

    /**
     * to be used when you want the header to be re read in memory to 
     * update the defaultKeywordQuery - case of Preference change
     * MZ 
     */

    public void resetDescription() {
        setDefaultKeywordQueryToNull();
        myFile.updateKeywords(defaultKeywordQuery);
        setDescription();
    }

    public void setDescription() {
        if (getDefaultFileDescription() != null) {
            getDefaultFileDescription().setDescription(this);
        }
    }

    // **************************************************************************

    public String getArcfile() {
        if (isPipeProduct()) {
            return new String("");
        } else {
            return arcFile;
        }
    }

    // from here on, all the abstract method of OBComponent which do not
    // differ between Fits and TFits are implemented
    // ******************************************************************

    /**
     * @return The validity status of the file after performing the update.
     * Check the status of the file and return it.
     * if the status has changed, remove the current object from the
     * model and create a new FitsFile object, add it to the model.
     * @see org.eso.gasgano.DFSFile
     */
    public int update() {
        if (myFile.dataRefreshRequired()) {
            String currentProgId = null;
            String currentObId = null;
            String currentTelescopeId = null;
            // first we must remove the file from the model.
            boolean inModel = (getStatus() == DFSFile.FILE_VALID);
            if (hasIntermediate()) {
                InterContainer inter =
                    (InterContainer) getObservation(GasProp.TREE_DEFAULT);
                currentObId = inter.getObservation().getId();
                currentProgId = inter.getObservation().getProgram().getId();
            } else {
                currentProgId =
                    getObservation(GasProp.TREE_DEFAULT).getProgram().getId();
                // the TREE_DEFAULT mode is not relevant, anyone would do
                currentObId = getObservation(GasProp.TREE_DEFAULT).getId();
            }
            currentTelescopeId = myFile.getTelescope();

            init(myFile.getPathString());
            // insert this component if the file is still valid.
            if (getStatus() == DFSFile.FILE_VALID) {
                // add this OBComponent to the DataModel.
                if (currentProgId.equals(defaultKeywordQuery[0].getValue())
                    && currentObId.equals(defaultKeywordQuery[1].getValue())) {
                    // same prog/ob - do not change the DataModel.
                } else {
                    // must remove this from the DataModel and add again.
                    if (inModel) {
                        removeFromDataModel();
                    }
                    addToDataModel(
                        defaultKeywordQuery[0].getValue(),
                        defaultKeywordQuery[1].getValue(),
                        myFile.getTelescope());
                    DFSDataModel.getDataModel().fireDataModelChangeEvent(
                        new DataModelEvent(DataModelEvent.FILE_ADDED));
                }
            } else if (inModel) {
                // remove it from the model as it is no longer valid.
                removeFromDataModel();
            }
        }
        return getStatus();
    }

    /**
     * @return The id string which uniquely identifies this in the
     * DataModel.
     * NB: This is the original address with which the FitsFile was created,
     * not the resolved address returned from a File object.
     */
    public String getId() {
        return myFile.getAddress();
    }

    public DFSFile getSourceFile() {
        return myFile;
    }

    public int getStatus() {
        return myFile.getStatus();
    }

    /**
     * @return The JComponent for displaying this OB Component.
     */
    public JComponent getDisplayComponent() {
        JComponent rc = null;
        boolean freeMemory = true;
        if (myFile.headerInMemory()) {
            freeMemory = false;
        } else {
            if (myFile
                .readFitsHeaderAndExtensions(FitsFileDisplay.isShowExtensions())
                == DFSFile.FILE_NOT_FOUND) {

                if (!DFSDataModel.getDataModel().isUpdating()) {
                    new Thread(new Runnable() {
                        public void run() {
                            DFSDataModel.getDataModel().update();
                        }
                    }).start();
                }

                rc =
                    new JLabel(
                        "The file has been slyly removed from the filesystem",
                        SwingConstants.CENTER);
                return rc;
            }
        }
        String title = null;
        Keyword origFile = myFile.getKeyword(Keyword.ORIGFILE);
        if (origFile == null) {
            title =
                toString()
                    + "    <unknown orig name>	   "
                    + myFile.getClassification();
        } else {
            title =
                getId()
                    + "                "
                    + origFile.getValue()
                    + "	  "
                    + myFile.getClassification();
        }
        rc = new FitsFileDisplay(title, new FitsTableModel(myFile));
        if (freeMemory) {
            myFile.freeHeaderMemory();
        }
        return rc;
    }

    /**
     * Get the grouping id for this component.
     * @see org.eso.gasgano.datamodel.gui.OBComponent#getIntermediateGrouping()
     */
    public String getIntermediateGrouping() {
        return myFile.getIntermediateGrouping();
    }

    /**
     * @return boolean true if this frame "appears to be!" a 
     * calibration frame.
     */
    public boolean isCalibFrame() {
        return myFile.isCalibFrame();
    }

    public String getSignature() {
        return myFile.getSignature();
    }

    /**
     * we take for granted that the presence of the keyword
     * HIERARCH ESO PRO REC1 RAW1 NAME indicates
     * that the frame contains reduction block information
     */

    public boolean containsRbInfo() {
        return myFile.containsRbInfo();
    }

    public String getAssociatedRecipeName() {
        String rc = "";
        Keyword recId = myFile.getKeyword("PRO.REC1.ID");
        if (recId != null) {
            rc = recId.getValue();
            if (rc == null) {
                rc = "";
            }
        }
        return rc;
    }

    /**
     * this method only makes sense for reduced frames
     * containing RB information
     * @returns: a Vector of two vectors. The first Vector 
     * contains the names, the second one the signatures
     * If a Signature is not available, a fake one is inserted
     */

    public Vector getAssociatedMasterInfo() {
        String keyBaseName = "PRO.REC";
        String keyBaseNameRec = keyBaseName + "1.CAL";
        String keyName1 = keyBaseNameRec + "1.NAME";
        String keyName2 = keyBaseNameRec + "1.DATAMD5";

        Vector rc = new Vector();
        Vector masterNames = new Vector();
        Vector masterSignatures = new Vector();

        int frameCount = 1, recCount = 1;
        Keyword tmpKey = null;

        // interface! Sequence IS IMPORTANT!
        rc.addElement(masterNames);
        rc.addElement(masterSignatures);

        while ((tmpKey = myFile.getKeyword(keyName1)) != null) {

            while ((tmpKey = myFile.getKeyword(keyName1)) != null) {
                //System.out.println(tmpKey);
                masterNames.addElement(tmpKey.getValue());
                if ((tmpKey = myFile.getKeyword(keyName2)) != null) {
                    masterSignatures.addElement(tmpKey.getValue());
                } else {
                    masterSignatures.addElement(fakeSignature);
                }
                frameCount++;
                keyName1 =
                    keyBaseNameRec + String.valueOf(frameCount) + ".NAME";
                keyName2 =
                    keyBaseNameRec + String.valueOf(frameCount) + ".DATAMD5";
            }
            recCount++;
            keyBaseNameRec = keyBaseName + String.valueOf(recCount) + ".CAL";
            frameCount = 1;
            keyName1 = keyBaseNameRec + "1.NAME";
            keyName2 = keyBaseNameRec + "1.DATAMD5";
        }

        return rc;
    }

    /**
     * this method only makes sense for reduced frames
     * containing RB information
     */

    public Vector getAssociatedRawFrames() {
        String keyBaseName = "PRO.REC";
        String keyBaseNameRec = keyBaseName + "1.RAW";
        String keyName = keyBaseNameRec + "1.NAME";
        Vector rc = new Vector();
        int frameCount = 1, recCount = 1;
        Keyword tmpKey = null;
        while ((tmpKey = myFile.getKeyword(keyName)) != null) {

            while ((tmpKey = myFile.getKeyword(keyName)) != null) {
                //System.out.println(tmpKey);
                rc.addElement(tmpKey.getValue());
                frameCount++;
                keyName = keyBaseNameRec + String.valueOf(frameCount) + ".NAME";
            }
            recCount++;
            keyBaseNameRec = keyBaseName + String.valueOf(recCount) + ".RAW";
            frameCount = 1;
            keyName = keyBaseNameRec + "1.NAME";
        }

        return rc;
    }

    /**
     * this method has to be created because it would make no sense
     * to read the header and release the memory three times in a row
     * The responsability to release the memory is thus given 
     * to the customer class
     */

    public void freeHeaderMemory() {
        myFile.freeHeaderMemory();
    }

    public boolean precedes(KeywordContainerOBComponent other) {
        boolean rc = false;
        String keyMjd, otherKeyMjd = null;

        keyMjd = myFile.getMjdObs();
        otherKeyMjd = ((FitsFile) other.getSourceFile()).getMjdObs();
        if (keyMjd.equals("")) {
            rc = false;
        } else {
            if (otherKeyMjd.equals("")) {
                rc = true;
            } else {
                if (keyMjd.compareTo(otherKeyMjd) <= 0) {
                    rc = true;
                    //System.out.println(getId() + " precedes "+other.getId());
                } else {
                    rc = false;
                }
            }
        }
        return rc;
    }

} // end of class HasHeaderOBComponent.java

// end of file HasHeaderOBComponent.java
