/**
 * @author N. Kornweibel
 * @version 1.0
 * Editor panel for a KeywordTable.
 */

package org.eso.gasgano.gui;

import java.awt.BorderLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.InputEvent;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;

import javax.swing.DefaultCellEditor;
import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.SwingUtilities;
import javax.swing.event.CellEditorListener;
import javax.swing.event.ChangeEvent;
import javax.swing.table.TableCellEditor;

public class KeywordListEditor
    extends JPanel
    implements ActionListener, CellEditorListener, MouseListener {

    private KeywordTable kwTable = null;
    private JTable table = null;
    private JButton insertRow = null;
    private JButton deleteRow = null;
    // does this editor write the file to disk.
    private boolean readOnlyMode = false;
    private static final String insertRowLabel = "Insert Row";
    private static final String deleteRowLabel = "Delete Row";

    public KeywordListEditor(KeywordTable kwt) {
        super(new BorderLayout());
        kwTable = kwt;
        table = new JTable(kwTable);
        table.setCellSelectionEnabled(false);
        table.getDefaultEditor(table.getColumnClass(0)).addCellEditorListener(
            this);
        table.getDefaultEditor(table.getColumnClass(1)).addCellEditorListener(
            this);
        insertRow = new JButton(insertRowLabel);
        // special case
        insertRow.addMouseListener(this);

        deleteRow = new JButton(deleteRowLabel);
        deleteRow.addActionListener(this);
        JPanel lowerPanel = new JPanel();
        lowerPanel.add(insertRow);
        lowerPanel.add(deleteRow);
        add(new JScrollPane(table), BorderLayout.CENTER);
        add(lowerPanel, BorderLayout.SOUTH);
        // disable/enable the delete button.
        deleteRow.setEnabled(kwTable.getRowCount() != 0);
    }

    public void setReadOnly(boolean readOnly) {
        readOnlyMode = readOnly;
    }

    public boolean isReadOnly() {
        return readOnlyMode;
    }

    public static void main(String[] args) {
        if (args.length < 1) {
            System.out.println("usage: <keyword file>");
            System.exit(0);
        }
        JFrame frame = new JFrame("Example keyword list: " + args[0]);
        KeywordTable kwt = new KeywordTable(args[0]);
        if (!kwt.readFile()) {
            System.out.println("Error reading in file: " + args[0]);
        }
        KeywordListEditor editor = new KeywordListEditor(kwt);
        frame.addWindowListener(new WindowAdapter() {
            public void windowClosing(WindowEvent e) {
                System.exit(0);
            }
        });
        frame.getContentPane().add(editor);
        frame.setSize(400, 300);
        frame.setVisible(true);
    }

    public KeywordTable getKeywordTable() {
        return kwTable;
    }

    // implementation of the ActionListener interface
    public void actionPerformed(ActionEvent a) {
        if (a.getSource() == deleteRow) {
            deleteSelectedRow();
        }
    }

    public void finishEditing() {
        if (table.isEditing()) {
            final int row = table.getSelectedRow();
            TableCellEditor comp = table.getCellEditor();
            if (comp != null && comp instanceof DefaultCellEditor) {
                ((DefaultCellEditor) comp).stopCellEditing();
            }
            table.getSelectionModel().setSelectionInterval(row, row);

        }
    }

    private void deleteSelectedRow() {
        finishEditing();
        int selRow = table.getSelectedRow();
        int rowCountNow = table.getRowCount();
        if (selRow < 0) {
            selRow = table.getRowCount() - 1;
        }
        if (selRow > -1) {
            kwTable.removeRow(selRow);
            if (!isReadOnly() && !kwTable.writeFile()) {
                userMessage("Error saving file: " + kwTable.getTablePath());
            }
            if (selRow >= rowCountNow-1) {
                // move the selected row to the new end of the table.
                selRow = rowCountNow-2;
            }
            if (selRow >= 0) {
                final int row = selRow;
                SwingUtilities.invokeLater(new Runnable() {
                    public void run() {
                        table.getSelectionModel().setSelectionInterval(row, row);
                    }});
            } else {
                table.clearSelection();
            }
        }
        // disable/enable the delete button.
        deleteRow.setEnabled(kwTable.getRowCount() != 0);
    }

    private void insertBlankRow(boolean after) {
        finishEditing();
        int selRow = table.getSelectedRow();
        if (selRow < 0) {
            selRow = table.getRowCount() - 1;
        }
        if (selRow < 0) {
            after = true;
        }
        if (after) {
            kwTable.insertRow(selRow + 1);
        } else {
            kwTable.insertRow(selRow);
            selRow++;
        }
        final int row = selRow;
        SwingUtilities.invokeLater(new Runnable() {
            public void run() {
                table.getSelectionModel().setSelectionInterval(row, row);
            }});
        if (!isReadOnly() && !kwTable.writeFile()) {
            userMessage("Error saving file: " + kwTable.getTablePath());
        }
        // disable/enable the delete button.
        deleteRow.setEnabled(kwTable.getRowCount() != 0);
    }

    public void editingCanceled(ChangeEvent e) {
    }

    public void editingStopped(ChangeEvent e) {
        if (!isReadOnly() && !kwTable.writeFile()) {
            userMessage("Error saving file: " + kwTable.getTablePath());
        }
    }

    public void mouseClicked(MouseEvent e) {
    }

    public void mouseEntered(MouseEvent e) {
    }

    public void mouseExited(MouseEvent e) {

    }

    public void mousePressed(MouseEvent e) {
        if ((e.getModifiers() & InputEvent.BUTTON3_MASK) != 0) {
            insertBlankRow(false);
        }
        if ((e.getModifiers() & InputEvent.BUTTON1_MASK) != 0) {
            insertBlankRow(true);
        }
    }

    public void mouseReleased(MouseEvent e) {
    }

    public void userMessage(String msg) {
        JOptionPane.showMessageDialog(this, msg);
    }
}
