/**
 * @author N. Kornweibel
 * @version 1.0
 */

package org.eso.gasgano.gui;

import java.awt.BorderLayout;
import java.awt.Menu;
import java.awt.MenuBar;
import java.awt.MenuItem;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.io.File;

import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JOptionPane;

import org.eso.dfs.gui.SwingFileChooser;
import org.eso.gasgano.datamodel.filesystem.DFSDataModel;
import org.eso.gasgano.properties.PropertyDB;
 
public class KeywordListEditorFrame extends JFrame implements ActionListener {

    private KeywordListEditor editorPanel = null;
    private KeywordTable kwTable = null;
    private static final String newItem = "New";
    private static final String openItem = "Open";
    private static final String saveAsItem = "Save As";
    private static final String quitItem = "Quit";
    private JFileChooser fileChooser = null;

    public static void main(String[] args) {
        KeywordListEditorFrame myApp = null;
        if (args.length > 0) {
            myApp = new KeywordListEditorFrame(args[0]);
        } else {
            myApp = new KeywordListEditorFrame();
        }
        myApp.addWindowListener(new WindowAdapter() {
            public void windowClosing(WindowEvent e) {
                System.exit(0);
            }
        });
        myApp.pack();
        myApp.setVisible(true);
    }

    public KeywordListEditorFrame() {
        kwTable = new KeywordTable();
        init();
        setTitle("Keyword List Editor (no file loaded)");
    }

    public KeywordListEditorFrame(String kwFile) {
        kwTable = new KeywordTable(kwFile);
        init();
        setTitle("Keyword List Editor: "+kwFile);
    }

    private void init() {
        setDefaultCloseOperation(JFrame.DO_NOTHING_ON_CLOSE);
        editorPanel = new KeywordListEditor(kwTable);
        getContentPane().add(editorPanel, BorderLayout.CENTER);
        addMenu();
        addWindowListener(new WindowAdapter() {
            public void windowClosing(WindowEvent e) {
                closeEditor();
            }
        });
    }

    private void addMenu() {
        MenuBar mb = new MenuBar();
        Menu m = new Menu("File");
 
        MenuItem mi = null;
 
        mi = new MenuItem(openItem);
        mi.addActionListener(this);
        m.add(mi);
 
        mi = new MenuItem(newItem);
        mi.addActionListener(this);
        m.add(mi);
 
        mi = new MenuItem(saveAsItem);
        mi.addActionListener(this);
        m.add(mi);
 
        mi = new MenuItem(quitItem);
        mi.addActionListener(this);
        m.add(mi);

        mb.add(m);
        setMenuBar(mb);
    }

    public void closeEditor() {
        editorPanel.finishEditing();
        if (kwTable.getTablePath() != null
            || !kwTable.isEdited()
            || (kwTable.isEdited() 
                && userQuestion("The keyword list is not saved to a file.\n"
                    +"Are you sure you want to close the editor?", "Warning"))) { 
            kwTable.resetEditFlag();
            setVisible(false);
        }
    }

    // implementation of the ActionListener interface
    public void actionPerformed(ActionEvent a) {
        if (a.getActionCommand() == quitItem) {
            closeEditor();
        } else if (a.getActionCommand() == saveAsItem) {
            saveAsFile();
        } else if (a.getActionCommand() == openItem) {
            openFile();
        } else if (a.getActionCommand() == newItem) {
            newFile();
        }
    }

    private JFileChooser getFileChooser() {
        if (fileChooser == null) {
			DFSDataModel dm = DFSDataModel.getDataModel();
			String kwDir = PropertyDB.getInstance().getProperty("KEYWORDLIST_DIR");
            fileChooser = new SwingFileChooser(kwDir);
        } else {
            fileChooser.rescanCurrentDirectory();
        }
        return fileChooser;
    }

    private void openFile() {
        JFileChooser openChooser = getFileChooser();
        openChooser.setFileSelectionMode(JFileChooser.FILES_ONLY);
        openChooser.setDialogType(JFileChooser.OPEN_DIALOG);
        openChooser.setDialogTitle("Select keyword list file");
        openChooser.setApproveButtonText("Load");
        openChooser.setMultiSelectionEnabled(false);
        if (openChooser.showOpenDialog(this) == JFileChooser.APPROVE_OPTION) {
            if (openChooser.getSelectedFile() != null) {
                String selectedFile = openChooser.getSelectedFile().getPath();
                if (!kwTable.readFile(selectedFile)) {
                    userMessage("Errors reading: "+selectedFile);
                }
                setTitle("Keyword List Editor: "+selectedFile);
            }
        }
        kwTable.resetEditFlag();
    }

    private void newFile() {
        for (int i = kwTable.getRowCount()-1; i >= 0; i--) {
            kwTable.removeRow(i);
        }
        kwTable.insertRow(0);
        writeToNewFile("Enter new keyword list name", "OK");
        kwTable.resetEditFlag();
    }

    private void saveAsFile() {
        editorPanel.finishEditing();
        writeToNewFile("Save keyword list in file", "Save");
        kwTable.resetEditFlag();
    }

    private void writeToNewFile(String dialogTitle, String approveButton) {
        JFileChooser saveChooser = getFileChooser();
        saveChooser.setFileSelectionMode(JFileChooser.FILES_ONLY);
        saveChooser.setDialogType(JFileChooser.SAVE_DIALOG);
        saveChooser.setDialogTitle(dialogTitle);
        saveChooser.setApproveButtonText(approveButton);
        saveChooser.setMultiSelectionEnabled(false);
        if (saveChooser.showSaveDialog(this) == JFileChooser.APPROVE_OPTION) {
            if (saveChooser.getSelectedFile() != null) {
                String selectedFile = saveChooser.getSelectedFile().getPath();
                File newFile = new File(selectedFile);
                boolean save = true;
                try {
                    if (!newFile.exists()
                        || JOptionPane.showConfirmDialog(this,
                        "Overwrite "+selectedFile+"?", selectedFile+" exists",
                        JOptionPane.OK_CANCEL_OPTION)
                             == JOptionPane.OK_OPTION) {
                        save = kwTable.writeFile(selectedFile);
                    }
                } catch (SecurityException e) {
                    save = false;
                }
                if (!save) {
                    userMessage("Errors writing: "+selectedFile);
                } else {
                    if (kwTable.getTablePath() != null) {
                        setTitle("Keyword List Editor: "
                            +kwTable.getTablePath());
                    }
                }
            }
        }
    }

    public void userMessage(String msg) {
        JOptionPane.showMessageDialog(this, msg);
    }
    
    public boolean userQuestion(String msg, String title) {
        return (JOptionPane.showConfirmDialog(this, msg, title, JOptionPane.OK_CANCEL_OPTION)
                == JOptionPane.OK_OPTION);
    }
}
