/**
 * @author N. Kornweibel
 * @version 1.0
 *
 * The Keyword data object, defining the Keyword in the DFS.
 * A Keyword contains a name and a value. Both a Strings. The name
 * is used for sorting.
 */

package org.eso.gasgano.keyword;

import java.io.Serializable;

import org.eso.dfs.util.KeywordToken;

public class Keyword implements Serializable {

    // fingerprint of original Keyword class in order to maintain
    // Object stream compatibility.
    static final long serialVersionUID = 1883435576112404647L;

    private String name = null;
    private String value = null;

    /*
     * Global keyword definitions used throughout the application.
     */

    public static final String hierarchKeywordPrefix = "HIERARCH ESO";

    /**
     * Name of the keyword identifying the instrument.
     */
    public static final String INSTRUMENT = "INSTRUME";

    /**
     * Name of the keyword identifying the telescope.
     */
    public static final String TELESCOP = "TELESCOP";

    /**
     * Name of the keyword identifying the archived filename
     */
    public static final String ARCFILE = "ARCFILE";

    /**
     * Name of the keyword identifying the program id.
     */
    public static final String PROG_ID = "OBS.PROG.ID";

    /**
     * Name of the keyword identifying the observation id.
     */
    public static final String OBS_ID = "OBS.ID";

    /**
     * Name of the keyword identifying the observation name.
     */
    public static final String OBS_NAME = "OBS.NAME";

    /**
     * Name of the keyword identifying the date and time of the observation.
     */
    public static final String DATE = "DATE";

    /**
     * Name of the keyword identifying the Principal Investigator.
     */
    public static final String PI = "PI-COI";

    /**
     * Name of the keyword, whose existance indicates a pipeline product file.
     */
    public static final String PIPELINE_IDENT = "PRO.CATG";
    
    /**
     * Name of the keyword that may indicate that the file is a calibration
     * frame.
     */
    public static final String DRPCATG = "DPR.CATG";

    /**
     * Name of the keyword storing the MD5 signature
     */
    public static final String MD5SIG = "DATAMD5";

    /**
     * Name of the keyword identifying the original file name.
     */
    public static final String ORIGFILE = "ORIGFILE";
    
    /**
     * Name of the keyword identifying that the file contains extensions.
     */
    public static final String EXTENSION = "EXTEND";
    
    /**
     * Name of the keyword used to identify the number of axis in the image
     */
    public static final String NAXIS = "NAXIS";
    
    /**
     * Name of the prefix of the keywords idenitying the pixel size of each
     * axis in the image (e.g. if NAXIS is 2, there should be a NAXIS1 and NAXIS2
     * keyword.
     */
    public static final String AXIS = NAXIS;
    
    /**
     * Name of the keyword that identifies the size of the pixel speicified in
     * the NAXIS keywords.
     */
    public static final String BITPIX = "BITPIX";
    
    /**
     * Name of the first keyword in an extended keyword block
     */
    public static final String EXTENDSTART = "XTENSION";
    
    /**
     * Name of the first keywork in the standard keyword block
     * NB: may also be used instead of EXTENDSTART....
     */
    public static final String SIMPLESTART = "SIMPLE";

    public static final int METAKEYWORD_SEPARATOR = '_';
    public static final int DASH_SEPARATOR = '-';

    /**
     * Name of the Meta Keyword for the file Classification.
     */
    public static final String CLASSIFICATION_METAKEYWORD = "CLASSIFICATION";

    /**
     * Name of the Meta Keyword for the file path.
     */
    public static final String FILE_PATH_METAKEYWORD = "FILE_PATH";

    /**
     * Name of the Meta Keyword for the file Classification.
     * Possible values are: "T" and "F".
     */
    public static final String PIPE_PRODUCT_METAKEYWORD = "PIPE_PRODUCT";
    
    /**
     * Name of the Meta Keyword added to the FITS file extensions to indicate
     * the start of and extension block when displaying all keywords.
     * The FiteFileDisplay uses this information to know the end of the header 
     * and start of a keyword block.
     * The value of such a keyword is typically the set with
     * FitsExtKeywordBlock.generateExtensionName().
     */
    public static final String EXTENSION_METAKEYWORD = "EXTENSION";
    
    /**
     * The keyword identifying name of the extension (optional).
     */
    public static final String EXTENSION_NAME = "EXTNAME";
    
    /**
     * Keyword used to differentiate between different extensions of the same
     * name (optional).
     */
    public static final String EXTENSION_VERSION = "EXTVER";
    
    /**
     * Keyword used to define the RB product name.
     */
    public static final String RBPRODUCT_NAME = "PRO.REC1.RAW1.NAME";
    
    /**
     * @param n The name of this keyword, can not be null.
     * @param v The keyword value, maybe null.
     */
    public Keyword(String n, String v) {
        setName(n);
        setValue(v);
    }
    
    /**
     * Build a Keyword from a Keyword token.
     * @param kwt
     */
    public Keyword(KeywordToken kwt) {
        setName(kwt.getName());
        setValue(kwt.getValue());
    }

    public static void main(String args[]) {
        System.out.println("valid keyword name: FRED: "+validName("FRED"));
        System.out.println("valid keyword name: F RED: "+validName("F RED"));
        System.out.println("valid keyword name: F-RED: "+validName("F-RED"));
        System.out.println("valid keyword name: F_RED: "+validName("F_RED"));
        System.out.println("valid keyword name: 99X: "+validName("99X"));
        System.out.println("valid keyword name: TPL.ID: "+validName("TPL.ID"));
        System.out.println("valid keyword name: tpl.ID: "+validName("tpl.ID"));
        System.out.println("valid keyword name: HIERARCH ESO TPL ID: "
            +validName("HIERARCH ESO TPL ID"));
        for (int i = 0; i < args.length; i++) {
            System.out.println(args[i]);
            System.out.println("    Full Name: "
                +Keyword.convertDotNameToFull(args[i]));
            System.out.println("    Dot Name: "
                +Keyword.convertFullNameToDot(args[i]));
        }
    }

    /**
     * @param dotName The shortened 'dot' form of the keyword name.
     * @return The expanded name for the Keyword.
     */
    public static String convertDotNameToFull(String dotName) {
        String rc = dotName;
        if (rc.indexOf('.') != -1) {
            rc = hierarchKeywordPrefix+" "+dotName.replace('.', ' ');
        }
        return rc;
    }

    /**
     * @param fullName The full/expanded name for the Keyword.
     * @return The shortened 'dot' form of the keyword name.
     */
    public static String convertFullNameToDot(String fullName) {
        String rc = null;
        if (fullName.startsWith(hierarchKeywordPrefix)) {
            try {
                // remove the prefix and the space after the prefix.
                rc = fullName.substring(hierarchKeywordPrefix.length()+1);
                rc = rc.replace(' ', '.');
            } catch (StringIndexOutOfBoundsException e) {
                // substring failed - keyword does not require expansion.
                rc = fullName;
            }
        } else {
            rc = fullName;
        }
        return rc;
    }


    /**
     * @return The (shortened/dot form) name of this keyword.
     */
    public String getName() {
        return name;
    }

    /**
     * @return The full name of this keyword.
     */
    public String getFullName() {
        return convertDotNameToFull(name);
    }

    /**
     * @return The value of this keyword as a String.
     */
    public String getValue() {
        return value;
    }

    /**
     * @param n The name of the Keyword.
     */
    public void setName(String n) {
        name = convertFullNameToDot(n);
    }

    /**
     * @param v The value of this keyword. Setting the value will remove 
     * any bounding quotes before assigning the value. Ie 'fred' will be
     * stored as fred.
     */
    public void setValue(String v) {
        if (v != null) {
            value = v.trim();
            int startIndex = value.indexOf('\'');
            int endIndex = value.lastIndexOf('\'');
            try {
                if (endIndex < 0 || endIndex == startIndex) {
                    value = value.substring(startIndex+1);
                } else {
                    value = value.substring(startIndex+1, endIndex);
                }
                value = value.trim();
            } catch (StringIndexOutOfBoundsException e) {
                System.out.println(getName()+" "+v+" : "+e.toString());
            }
        } else {
            value = v;
        }
    }

    /**
     * @return The value as a Float.
     * @throws NumberFormatException if the value cannot be converted 
     * to a Float.
     */
    public Float getNumericValue() throws NumberFormatException {
        return Float.valueOf(getValue());
    }


    /**
     * @param anObject The object to be used in the comparison.
     * @return True if the object string value is the same as the name of
     * this keyword.
     */
    public boolean equals(Object anObject) {
        boolean rc = false;
        if (anObject instanceof String) {
            rc = name.equals(convertFullNameToDot((String)anObject));
        }
        return rc;
    }

    /**
     * @return A String representing this keyword: "name = value".
     */
    public String toString() {
        return name.toString()+" = "+value.toString();
    }

    /**
     * A keyword name is valid if it is upper case and contains
     * only alphanumeric characters, dots "." and '_'.
     * It allows MetaKeywords (containing '_') which are not legal
     *  Keyword - oh well!
     */
    public static boolean validName(String kwName) {
        boolean rc = true;
        int i = 0;
        int len = kwName.length();
        int seperator = '.';
        if (kwName.startsWith(hierarchKeywordPrefix)) {
            seperator = ' ';
        }
        while (rc && i < len) {
            try {
                char c = kwName.charAt(i++);
                if ((c >= 'A' && c <= 'Z')||(c >= '0' && c <= '9')
                        || (c == seperator)|| ( c == DASH_SEPARATOR) ) {
		  // (c == METAKEYWORD_SEPARATOR) || 
                } else {
                    rc = false;
                }
            } catch (StringIndexOutOfBoundsException e) {
                i = len;
            }
        }
        return rc;
    }
} // end of class Keyword.
// end of file Keyword.java.
