/**
 * @author N. Kornweibel
 * @version 1.0
 * The like operator equaivalence with the option of using a "*" wild
 * card in the RHS String.
 */

package org.eso.gasgano.keyword;

import java.io.Serializable;
import java.util.Enumeration;
import java.util.Vector;

public class LikeKeywordExpr extends BinaryKeywordExpr {

    static final long serialVersionUID = -4276010844095737983L;
    
    public static final String opString = "like";
    
    private boolean argumentsValidated = false;
    private String exceptionMessage = null;
    private Vector matchTokens = null;
    public static final char WILD_CARD = '*';

    /**
     * @param l The left hand value.
     * @param r The right hand value.
     */
    public LikeKeywordExpr(Object l, Object r) {
        super(l, r);
        matchTokens = new Vector();
    }

    // eval() should never be called. They are defined as they are
    // required by the BinaryKeywordExpr base class.
    public boolean eval(float l, float r) {
        return false;
    };
 
    // eval() should never be called. They are defined as they are
    // required by the BinaryKeywordExpr base class.
    public boolean eval(String l, String r) {
        return false;
    }


    /**
     * The left argument must be matched with the right argument, where
     * the right argument must be a string and may contain "*"
     * characters to indicate wild cards.
     */
    public boolean evaluate(KeywordSet s)
            throws ExpressionEvaluationException {
        boolean rc = true;
        if (!argumentsValidated) {
            if (!isStringCompatible(getLeftValue()))  {
                exceptionMessage = new String
                    ("Left argument could not be read as a character string.");
            }
            if (getRightValue() instanceof String == false) {
                exceptionMessage 
                    = new String("Right argument is not a character string.");
            } else {
                // parse the match string into itse components.
                String matchString = (String)getRightValue();
                int offset = 0;
                int matchLen = matchString.length();
                try {
                    while (offset != matchLen) {
                        // System.out.println("str: "+matchString +" "+offset);
                        if (matchString.charAt(offset) == WILD_CARD) {
                            matchTokens.addElement(new WildCard());
                            offset++;
                        } else {
                            int tokenEnd = matchString.indexOf(WILD_CARD,
                                offset);
                            if (tokenEnd == -1) {
                                tokenEnd = matchString.length();
                            }
                            String token =
                                matchString.substring(offset, tokenEnd);
                            if (token.length() > 0) {
                                matchTokens.addElement(token.toUpperCase());
                            }
                            offset = tokenEnd;
                        }
                    }
                } catch (StringIndexOutOfBoundsException e) {
                    exceptionMessage = 
                        new String("Invalid match string: "+matchString+".");
                }
            }
            argumentsValidated = true;
        }
        if (exceptionMessage != null) {
            throw new ExpressionEvaluationException(exceptionMessage);
        }
        // both arguments appear ok..evaluate the expression.

        resolveKeyword(getLeftValue(), s);
        String val = getString(getLeftValue());
        if (matchTokens.size() == 0 && val.length() > 0) {
            rc = false;
        }
        // printTokens();
        Enumeration e = matchTokens.elements();
        boolean exactTokenMatchRequired = true;
        int offset = 0;
        while (rc && e.hasMoreElements()) {
            Object tok = e.nextElement();
            if (tok instanceof WildCard) {
                exactTokenMatchRequired = false;
            } else {
                String tokStr = (String)tok;
                int matchIndex = val.indexOf(tokStr, offset);
                if (matchIndex == -1) {
                    // no match found.
                    rc = false;
                } else if (matchIndex != offset && exactTokenMatchRequired) {
                    // match found but not at this offset.
                    rc = false;
                } else {
                    // valid match found.
                    offset = matchIndex+1+tokStr.length();
                    exactTokenMatchRequired = true;
                }
            }
        }
        if (offset < val.length() && exactTokenMatchRequired) {
            // characters left at the end and no wild card read.
            rc = false;
        }
        return rc;
    }

    public void printTokens() {
        Enumeration e = matchTokens.elements();
        System.out.print("Like Tokens:");
        while (e.hasMoreElements()) {
            Object o = e.nextElement();
            if (o instanceof WildCard) {
                System.out.print(" *");
            } else {
                System.out.print(" "+o.toString());
            }
        }
        System.out.println("");
    }

    public String getOpString() {
        return opString;
    }

    class WildCard implements Serializable {
        // nothing other than an object to mark a wild card.
    }
    /**
     * @see org.eso.gasgano.keyword.DefaultBoolKeywordExpr#copy()
     */
    public DefaultBoolKeywordExpr copy() {
        DefaultBoolKeywordExpr rc = new LikeKeywordExpr(getLeftValue(), getRightValue());
        return rc;
    }
}
