/**
 * @author N. Kornweibel
 * @version 1.0
 * RuleTable - collection of classification rules
 * 
 * TODO: this class requires swing as it is an AbstractTableModel - ideally
 * it should be a datatable without swing bindings.
 */

package org.eso.gasgano.keyword;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.io.Serializable;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Vector;

import javax.swing.table.AbstractTableModel;

import org.eso.dfs.io.UpgradingObjectInputStream;
import org.eso.gasgano.properties.PropertyDB;

public class RuleTable extends AbstractTableModel implements Serializable {

    // NB: the rules member is written to the ObjectStream.
    private Vector rules = new Vector();
    private Hashtable instRuleSet = new Hashtable();
    private static String columnNames[] =
        { "Instrument", "Classification", "Rule" };
    private String ruleTablePath = null;

    public RuleTable(String filePath) {
        ruleTablePath = filePath;
    }

    /**
     * @param c The column number.
     * @return The name of the column requirested.
     * Part of the AbstractTableModel extension.
     */
    public String getColumnName(int c) {
        return columnNames[c];
    }

    public String getTablePath() {
        return ruleTablePath;
    }

    /**
     * @return The number of columns in the table - always 3.
     */
    public int getColumnCount() {
        return 3;
    }

    /**
     * @return The number of rows (rules) in the table.
     */
    public int getRowCount() {
        return rules.size();
    }

    /**
     * @param r The row number of the cell value required.
     * @param c The column number of the cell value required.
     * @return The Object represnting the value of the cell.
     */
    public Object getValueAt(int r, int c) {
        ClassificationRule rule = (ClassificationRule) rules.elementAt(r);
        String rc = null;

        if (c == 0) {
            rc = rule.getInstrument();
        } else if (c == 1) {
            rc = rule.getClassification();
        } else {
            rc = rule.getRule().toString();
        }
        return rc;
    }

    /**
     * @param column The column whose width is required.
     * @return The width of the requested column.
     */
    public int getColumnWidth(int column) {
        if (column == 2) {
            return 30;
        } else {
            return 10;
        }
    }

    /**
     * @return False - the table is never editable.
     */
    public boolean isCellEditable(int r, int c) {
        return false;
    }

    /**
     * @return An Enumeration of the rules in the table.
     */
    public Enumeration elements() {
        return rules.elements();
    }

    /**
     * @return An enumeration of the rules for a specific instrument
     * in the table.
     */
    public Enumeration elements(String instName) {
        Enumeration rc = null;
        Vector vec = (Vector) instRuleSet.get(instName);
        if (vec != null) {
            rc = vec.elements();
        } else {
            rc = new Vector().elements();
        }
        return rc;
    }

    /**
     * @param o The rule to be removed from the table.
     * @return The true if the rule was located and removed.
     */
    public final synchronized boolean removeElement(Object obj) {
        boolean rc = rules.removeElement(obj);
        if (rc) {
            // remove the element from the instRuleSet.
            ClassificationRule cr = (ClassificationRule) obj;
            Vector instrumentRules =
                (Vector) instRuleSet.get(cr.getInstrument());
            if (instrumentRules != null) {
                instrumentRules.removeElement(obj);
            }
        }
        fireTableDataChanged();
        return rc;
    }

    /**
     * @param index The number of the rule required.
     * @return The ClassificationRule at the requested element, or null if
     * the element is empty/out of range.
     */
    public ClassificationRule elementAt(int index) {
        ClassificationRule rc = null;
        try {
            rc = (ClassificationRule) rules.elementAt(index);
        } catch (ArrayIndexOutOfBoundsException e) {
            System.out.println(e.toString());
        }
        return rc;
    }

    /**
     * @param index The number of the rule to be removed.
     */
    public final synchronized void removeElementAt(int index) {
        try {
            ClassificationRule cr = (ClassificationRule) rules.elementAt(index);
            rules.removeElementAt(index);
            // remove the element from the instRuleSet.
            Vector instrumentRules =
                (Vector) instRuleSet.get(cr.getInstrument());
            if (instrumentRules != null) {
                instrumentRules.removeElement(cr);
            }
        } catch (ArrayIndexOutOfBoundsException e) {
            System.out.println(e.toString());
        }
        fireTableDataChanged();
    }

    /**
     * Add a classification rule to the table.
     * The obj parameters is assumed to be a ClassificationRule.
     * @param obj The rule to be added (assumed to be a ClassificationRule).
     */
    public final synchronized void addElement(Object obj) {
        ClassificationRule cr = (ClassificationRule) obj;
        rules.addElement(obj);
        // add the element from the instRuleSet.
        Vector instrumentRules = (Vector) instRuleSet.get(cr.getInstrument());
        if (instrumentRules == null) {
            instrumentRules = new Vector();
            instRuleSet.put(cr.getInstrument(), instrumentRules);
        }
        instrumentRules.addElement(obj);
        fireTableDataChanged();
    }

    /**
     * @param obj The rule to be added.
     * @param index The index where the rule is to be stored.
     */
    public final synchronized void setElementAt(Object obj, int index) {
        try {
            rules.setElementAt(obj, index);
        } catch (ArrayIndexOutOfBoundsException e) {
            System.out.println(e.toString());
        }
        fireTableDataChanged();
    }

    /**
     * @param selectedRule  The rule to be moved down
     */
    public final boolean moveElementDown(ClassificationRule selectedRule) {
        int index = rules.indexOf(selectedRule);
        if (index == rules.size() - 1) {
            return false;
        }
        ClassificationRule displacedRule =
            (ClassificationRule) rules.elementAt(index + 1);
        rules.setElementAt(displacedRule, index);
        rules.setElementAt(selectedRule, index + 1);

        if (selectedRule
            .getInstrument()
            .equals(displacedRule.getInstrument())) {
            Vector instrumentRules =
                (Vector) instRuleSet.get(selectedRule.getInstrument());
            if (instrumentRules != null) {
                int index2 = instrumentRules.indexOf(selectedRule);
                displacedRule =
                    (ClassificationRule) instrumentRules.elementAt(index2 + 1);
                instrumentRules.setElementAt(selectedRule, index2 + 1);
                instrumentRules.setElementAt(displacedRule, index2);
            } else {
                System.err.println("Internal error in RuleTable");
            }
        }
        fireTableDataChanged();
        return true;
    }

    /**
     * @param selectedRule  The rule to be moved up
     */
    public final boolean moveElementUp(ClassificationRule selectedRule) {
        int index = rules.indexOf(selectedRule);
        if (index == 0) {
            return false;
        }
        ClassificationRule displacedRule =
            (ClassificationRule) rules.elementAt(index - 1);
        rules.setElementAt(displacedRule, index);
        rules.setElementAt(selectedRule, index - 1);

        if (selectedRule
            .getInstrument()
            .equals(displacedRule.getInstrument())) {
            Vector instrumentRules =
                (Vector) instRuleSet.get(selectedRule.getInstrument());
            if (instrumentRules != null) {
                int index2 = instrumentRules.indexOf(selectedRule);
                displacedRule =
                    (ClassificationRule) instrumentRules.elementAt(index2 - 1);
                instrumentRules.setElementAt(selectedRule, index2 - 1);
                instrumentRules.setElementAt(displacedRule, index2);
            } else {
                System.err.println("Internal error in RuleTable");
            }
        }
        fireTableDataChanged();
        return true;
    }

    /**
     * Read a RuleTable in from a hard coded file name storing the Object
     * stream.
     * Depending on property settings, errors will be listed to stdout.
     * @return The RuleTable read, or null if the read fails.
     */
    public static RuleTable readFile(String tablePath) {
        RuleTable rc = null;
        try {
            rc = readFileEx(tablePath);
        } catch (ClassNotFoundException e) {
            if (PropertyDB
                .getInstance()
                .isDebug(PropertyDB.debugExpressionAndRules)) {
                System.out.println(
                    "Error reading file: " + tablePath + " " + e.toString());
            }
            rc = null;
        } catch (IOException e) {
            if (PropertyDB
                .getInstance()
                .isDebug(PropertyDB.debugExpressionAndRules)) {
                System.out.println(
                    "Error reading file: " + tablePath + " " + e.toString());
            }
            rc = new RuleTable(tablePath);
            rc.setup(new Vector());
        }
        return rc;
    }

    /**
     * Read and create a RuleTable from the given path. On error,
     * throw the given exceptions.
     * TODO: readFileEx() and readFile() should be modified to use
     * the importFileEx() method instead.  
     * @param tablePath path to the rule table
     * @return RuleTable RuleTable object read from the file.
     * @throws ClassNotFoundException on file read error.
     * @throws IOException on file read error.
     */
    public static RuleTable readFileEx(String tablePath)
        throws ClassNotFoundException, IOException {
        RuleTable rc = new RuleTable(tablePath);
        // Read in with an Upgrading Input Stream to handle
        // package renaming.
        // ObjectInputStream in =
        //    new ObjectInputStream(new FileInputStream(tablePath));
        ObjectInputStream in =
            new UpgradingObjectInputStream(
                new FileInputStream(tablePath),
                "org.eso.gasgano.",
                "org.eso.gasgano.keyword.");
        Vector ruleSet = (Vector) in.readObject();
        rc.setup(ruleSet);
        return rc;
    }
    
    /**
     * Read on the given rule file, adding the contents to this table.
     * Depending on PropertyDB settings, report errors to stdout.
     * @param tablePath
     */
    public void importFile(String tablePath) {
        try {
            importFileEx(tablePath);
        } catch (ClassNotFoundException e) {
            if (PropertyDB
                .getInstance()
                .isDebug(PropertyDB.debugExpressionAndRules)) {
                System.out.println(
                    "Error importing file: " + tablePath + " " + e.toString());
            }
        } catch (IOException e) {
            if (PropertyDB
                .getInstance()
                .isDebug(PropertyDB.debugExpressionAndRules)) {
                System.out.println(
                    "Error importing  file: " + tablePath + " " + e.toString());
            }
        }
    }

    /**
     * Import the given file into this table, throwing an exception on error
     * @param tablePath path to file to read on
     * @throws IOException on read/open error
     * @throws ClassNotFoundException on read error.
     */
    public void importFileEx(String tablePath) throws IOException, ClassNotFoundException {
        //ObjectInputStream in =
        //    new ObjectInputStream(new FileInputStream(tablePath));
        ObjectInputStream in =
            new UpgradingObjectInputStream(
                new FileInputStream(tablePath),
                "org.eso.gasgano.",
                "org.eso.gasgano.keyword.");
        Vector ruleSet = (Vector) in.readObject();
        Enumeration e = ruleSet.elements();
        while (e.hasMoreElements()) {
            addElement(e.nextElement());
        }
    }

    public void writeFile() throws IOException {
        ObjectOutputStream out =
            new ObjectOutputStream(new FileOutputStream(getTablePath()));
        out.writeObject(rules);
        out.close();
    }

    private void setup(Vector ruleSet) {
        Enumeration e = ruleSet.elements();
        rules = new Vector();
        instRuleSet = new Hashtable();
        while (e.hasMoreElements()) {
            addElement(e.nextElement());
        }

    }

    /**
     * Main method to load a file and list its contents to std out in the order read in.
     * @param args
     */
    public static void main(String[] args) {
        if (args.length != 1 || args[0].equals("-v")) {
            System.out.println(
                "RuleTable usage: [-v | <classification.rul file>]");
            System.exit(0);
        } else {
            File rulFile = new File(args[0]);
            if (rulFile.exists()) {
                if (rulFile.canRead()) {
                    RuleTable tbl = null;
                    try {
                        tbl = RuleTable.readFileEx(args[0]);
                    } catch (ClassNotFoundException e) {
                        System.out.println("Error reading file: "+e.toString());
                    } catch (IOException e) {
                        System.out.println("Error reading file: "+e.toString());
                    }
                    if (tbl != null) {
                        for (int i = 0; i < tbl.getRowCount(); i++) {
                            ClassificationRule rul = tbl.elementAt(i);
                            System.out.print(rul.getInstrument());
                            int p = rul.getInstrument().length();
                            while (p < 15) {
                                System.out.print(" ");
                                p++;
                            }
                            System.out.print(rul.getClassification());
                            p = rul.getClassification().length();
                            while (p < 20) {
                                System.out.print(" ");
                                p++;
                            }                            
                            System.out.println(rul.getRule().toString());
                        }
                    }
                
                } else {
                    System.out.println("Can not read " + args[0] + ".");
                    System.exit(0);
                }
            } else {
                System.out.println("File " + args[0] + " does not exist.");
                System.exit(0);
            }
        }
    }
}
