package org.eso.ohs.core.dbb.sql;

import java.io.Serializable;

import org.apache.log4j.Logger;

/**  
  * $Id: DbbSqlOperator.java,v 1.4 2005/01/21 16:22:30 tcanavan Exp $
  *
  * The DbbSqlOperator class represents SQL operators like
  * ">=", "like", "group by", etc.
  * 
  * <br>
  * <br>
  * @author Carlo Boarotto, ESO 20-Jul-1999
  * <p>
  * */
public class DbbSqlOperator implements Serializable {
    /**
     * Logger for this class
     */
    private static final Logger stdlog_ = Logger
            .getLogger(DbbSqlOperator.class);

    /** 
     * List of valid SQL operator types. <BR>
     * See source code for complete list of values.
     */
    // Don't change the order of the operators because it affects the
    // evaluation order: i.e. ">=" should be evaluated before ">"
    //
    protected final static String[] operators_ = {
	"not like",
	"like", 
	"!=",
	"=", 
	">=",
	"<=",
	">",
	"<", 
	"not in",
	"in",
	"ascending",
	"descending",
	"is", 
	"not",
        "between"
    };

    /** 
     * Array of sql complement operators.
     *
     * Note: elements must match one-by-one those of 
     *       operators_.
     */		
    protected final static String[] nOperators_ = {
	"like",
	"not like",
	"=",
	"!=",
	"<",
	">",
	"<=",
	">=", 
	"in",
	"not in",
	"descending", 
	"ascending",
	"not",
	"is",
    };

    /** List of "literal" operators. */
    protected final static String[] literalOps_ = {
	"like", 
	"not like",
	"in",
	"not in",
	"ascending",
	"descending",
	"is", 
	"not"
    };
    
    /** Operator current value, e.g. "like", "=", ">=", etc. */
    protected String value_ = null;

    //--------------------------------------------------
    // Constructors
    //--------------------------------------------------

    /** 
     * Build a DbbSqlOperator object.
     *
     * @param type    SQL operator type: must be one of the elements 
     *                of operators_
     *
     * @exception DbbInvalidSqlOperatorType If arg <em>type</em> does not
     *                                      translate to a valid operator.
     */
    public DbbSqlOperator(String value) 
	throws DbbInvalidSqlOperatorType {
      
	set(value.toLowerCase());
    }

    //----------------------------------------
    // Private Methods
    //----------------------------------------

    /** 
     * Set the operator value.
     *
     * @param value Operator value: must be one of the elements of
     *		  operators_
     *
     * @exception DbbInvalidSqlOperatorType If arg <em>value</em> does not
     *                                      translate to a valid operator.  
     */
    private void set(String value) 
	throws DbbInvalidSqlOperatorType { 

	if(!isValidSQLOperator(value)) {
	
	    String errmsg = "Invalid SQL operator: '" + value + "'\n" +
		"Should be one of: ";
      
	    for( int i = 0; i < operators_.length; i++) 
		errmsg = errmsg + "'" + operators_[ i ] + "' ";
	
	    throw new DbbInvalidSqlOperatorType( errmsg );
	}
      
	value_ = value;
    }

    //----------------------------------------
    // Public methods
    //----------------------------------------

    /** @return Operator value. */
    public String get() { return value_; }


    /**
     * @return A new operator object, the SQL complement of the current
     *         DbbSqlOperator instance.
     *
     * @exception DbbInvalidSqlOperatorType TBD
     * @exception DbbInternalError          TBD
     */
    public DbbSqlOperator not()
   
	throws DbbInvalidSqlOperatorType, DbbInternalError {

	for(int i=0; i<operators_.length; i++) {
	    if(value_.equals(operators_[i]))
		return (new DbbSqlOperator(nOperators_[i]));
	}

	throw new DbbInternalError("complement of '" + 
				   value_ + "' not found");
    }


	/**
	 * Method getSqlOperator
	 * @param strOp String
	 * @return DbbSqlOperator
	 */
	public static DbbSqlOperator getSqlOperator(String strOp) {
		DbbSqlOperator sqlOp = null;
		try {
			sqlOp = new DbbSqlOperator(strOp);
		} catch (Exception e) {
			stdlog_.warn("", e);
		}
		return sqlOp;
	}

	/**
	 * Method getSqlOperator
	 * @param strOp String
	 * @return DbbSqlOperator
	 */
	public static DbbSqlOperator getEqualSqlOperator() {		
		return getSqlOperator("=");
	}	
	/**
	 * Method getSqlOperator
	 * @param strOp String
	 * @return DbbSqlOperator
	 */
	public static DbbSqlOperator getNotEqualSqlOperator() {		
		return getSqlOperator("!=");
	}	
	/**
	 * Method getSqlOperator
	 * @param strOp String
	 * @return DbbSqlOperator
	 */
	public static DbbSqlOperator getLikeSqlOperator() {		
		return getSqlOperator("like");
	}	/**
	 * Method getSqlOperator
	 * @param strOp String
	 * @return DbbSqlOperator
	 */
	public static DbbSqlOperator getInSqlOperator() {		
		return getSqlOperator("in");
	}
	/**
	 * Method getSqlOperator
	 * @param strOp String
	 * @return DbbSqlOperator
	 */
	public static DbbSqlOperator getNotInSqlOperator() {		
		return getSqlOperator("not in");
	}

	/**
	 * Method getSqlOperator
	 * @param strOp String
	 * @return DbbSqlOperator
	 */
	public static DbbSqlOperator getLowerEqOperator() {		
		return getSqlOperator("<=");
	}
	/**
	 * Method getSqlOperator
	 * @param strOp String
	 * @return DbbSqlOperator
	 */
	public static DbbSqlOperator getGreaterEqOperator() {		
		return getSqlOperator(">=");
	}
    /**
     * Class method to validate values.
     *
     * @param value Value to be checked.
     *
     * @return Boolean value <code>true</code> if it is a valid operator
     *         value otherwise <code>false</code>.  
     */
    public static boolean isValidSQLOperator(String value) {

	for(int i=0; i<operators_.length; i++) {
	    if(value.equals(operators_[i]))
		return true;
	}

	return false;
    }


    /** @return A string giving the operator value. */
    public String toString() { return value_; }


    /** @return A list of valid operators. */
    public static String[] getOperatorValues() { return operators_; }


    /**
     * @return Boolean flag indicating if a operator string value is a
     * literal ("like", "is", "in", etc.) operator. 
     */
    public static boolean isLiteral(String s) {
	for(int i=0; i<literalOps_.length; i++) {
	    if(s.equals(literalOps_[i]))
		return true;
	}

	return false;
    }

    /**
     * Compares the current instance and a given operator for
     * equality.
     * @param op Sql operator to be compared
     * @return true if this operator is the same as the op argument;
     *	       false otherwise.
     */
    public boolean equals(DbbSqlOperator op) {

	if(op == null)
	    return false;

	String op_str = op.get();
	return (value_.equals(op_str));
    }

    /**
     * Compares the current instance and a given operator for
     * equality.
     * @param op_str Sql operator to be compared
     * @return true if this operator is the same as the op_str argument;
     *	       false otherwise.
     */
    public boolean equals(String op_str) {
	return (value_.equals(op_str));
    }
}
