package org.eso.ohs.core.utilities;

import java.io.Serializable;
import java.sql.SQLException;

/**
 * A wrapper-exception for input/output exceptions from StorageManager
 * subclasses.  Because there are several different kinds of
 * StorageManager, each producing different kinds of exceptions it
 * is necessary to use a wrapper, so that the higher levels can know
 * exactly which kind of exception to catch for an I/O error.
 * There is also a second constructor for errors generated within
 * the storage manager, i.e.\ not due to another component throwing
 * an exception.
 * 
 * <p>The original exception (if any) is accessible using the
 * getException() method.
 * 
 * <p>
 * $Id: ObjectIOException.java,v 1.1 2004/02/17 14:31:55 tcanavan Exp $
 */

public class ObjectIOException extends Exception {

    /** The original exception caught by the StorageManager, or null */
    private NestedInfo nestedException_;

    /**
     * Construct an exception that does <em>not</em> contain another
     * exception.
     * 
     * @param msg   explanatory message giving a reason for the exception.
     */
    public ObjectIOException(String msg) {
	super(msg);
    }

    /**
     * Construct an exception that contains another exception.
     * The containing exception has the same message as the original.
     * 
     * @param exc  	the original exception
     */
    public ObjectIOException(Throwable exc) {
	this(exc, exc.getMessage());
    }

    /**
     * Construct an exception that contains another exception.
     * The containing exception has the same message as the original.
     * 
     * @param exc  	the original exception
     */
    public ObjectIOException(Throwable exc, String message) {
	super(message);

	    // XXX Hack for SQL exceptions in case they're sent over RMI
	    //     Too much informatin was lost in transit.
	if (exc instanceof SQLException) {
	    SQLException e = (SQLException)exc;
	    NestedInfo nse = new NestedInfo();

	    nse.message = e.getMessage();
	    nse.sqlState = e.getSQLState();
	    nse.errorCode = e.getErrorCode();

	    e = e.getNextException();
	    nestedException_ = nse;

	    while (e != null) {
		nse.next      = new NestedInfo();
		nse           = nse.next;
		nse.message   = e.getMessage();
		nse.sqlState  = e.getSQLState();
		nse.errorCode = e.getErrorCode();

		e   = e.getNextException();
	    }
	}
	else if ( ! exc.getMessage().equals(message) ) {
	    NestedInfo nse = new NestedInfo();

	    nse.message      = exc.getMessage();
	    nse.sqlState     = null;
	    nestedException_ = nse;
	}

    }


    /**
     * Returns the exception generated by the original problem.
     * May be null.
     * 
     * return the original exception
     */
    public NestedInfo getExceptionInfo() {
	return nestedException_;
    }

    public class NestedInfo implements Serializable {
	public String message;
	public String sqlState;
	public int errorCode;
	public NestedInfo next;
    }

}

