package org.eso.ohs.persistence.dbase.phase1;
import java.sql.Connection;
import java.sql.SQLException;

import org.apache.log4j.Logger;
import org.eso.ohs.core.utilities.DebugUtils;
import org.eso.ohs.core.utilities.ObjectIOException;
import org.eso.ohs.dfs.ObservingRun;
import org.eso.ohs.dfs.Readme;
import org.eso.ohs.dfs.StorableObject;
import org.eso.ohs.persistence.Media;
import org.eso.ohs.persistence.ObjectNotFoundException;
import org.eso.ohs.persistence.dbase.DbaseStorageMgr;
import org.eso.ohs.persistence.dbase.phase2.DbaseHandlerOBTargetConstrainInfo;
import org.eso.ohs.persistence.dbase.phase2.DbaseHandlerQueue;
import org.eso.ohs.persistence.dbase.phase2.DbaseHandlerTimeInterval;
import org.eso.ohs.persistence.dbase.phase2.DbaseHandlerRunProperties;
/**
 * The storage manager class for files stored in the Phase 1 database.
 * the internal flow for the registry  when the constructor is called is:
 *  OpcDbaseMgr() --> DbaseStorageMgr() --> OpcDbaseMgr:createRegistry() -->
 * Phase1HandlerRegistry() --> HandlerRegistry() -->  
 * Phase1HandlerRegistry:initialiseRegistry() and here you are where the 
 * handler register themselves and declare which class they manage 
 * (with DbaseIOHandler.getManagedClass()). After a handler and its 
 * managed class are registered (as seen above), when I call getObj
 * with the object type, I get the object through the handler.
 * 
 * <p>
 * $Id: OpcDbaseMgr.java,v 1.6 2005/09/15 09:22:13 ddorigo Exp $
 * 
 * <br>
 * @author Bob Kemp
 * <br>ChangeLog: cmavros 22Apr02 1195/P2PP Validation of OBs through P2PP AppServer.
 * 
 * @see org.eso.ohs.persistence.dbase.DbaseStorageMgr
 * @see org.eso.ohs.persistence.StorageManager
 * @see org.eso.ohs.persistence.ObjectManager
 */
public class OpcDbaseMgr extends DbaseStorageMgr {
	static OpcDbaseMgr singleton_ = null;
	private static Logger stdlog_ = Logger.getLogger(OpcDbaseMgr.class);
	/**
	 * Default constructor
	 */
	public OpcDbaseMgr() {
		super(Media.PHASE1_DB);
	}
	/** here is called the initialise registry. 
	 * This method is called by the DbaseStorageManager constructor
	 */
	protected void createRegistry() {
		registry_ = new Phase1HandlerRegistry();
	}
	/**
	 * Returns the cached JDBC database Connection object creating
	 * it first, if necessary.
	 * 
	 * @return  JDBC Connection object
	 * 
	 * @exception SQLException
	 *		    an error occurred during the dbase operation
	 * @exception ObjectIOException
	 *		    an error occured during an access to a business object
	 */
	protected Connection acquireConnection()
		throws ObjectIOException, SQLException {
		ObservingRun run = new ObservingRun();
		return acquireConnection(run.getClass());
	}
	/**
	 * Check in the specified objects in a single atomic operation.
	 * 
	 * @param	usos	Descriptions of each StorableObject to be checked in.
	 * 
	 * @return		array of error strings.
	 *			If length = 0, no errors.
	 */
	public void checkIn(MultiObjSpec[] moi) throws ObjectIOException {
		throw new ObjectIOException("Operation not supported in this database");
	}
	/**
	 * Do a EVM verification test for the specified objects in a single atomic operation.
	 * 
	 * @param	usos	Descriptions of each StorableObject to do EVM verification
	 * 
	 * @return boolean if OB passed EVM test	
	 */
	public boolean verifyObject(MultiObjSpec[] moi) throws ObjectIOException {
		return true;
	}
	/**
	 * Check out the ObsBlk or CalBlk with the specified id plus
	 * any attached objects, ie Target, ObsDescr, etc.
	 * 
	 * @param	id	id of the ObsBlk or CalBlk to be checked out
	 * 
	 * @return		CheckOutStatus object that contains a return
	 *			status and (if successful) references to the
	 *			checked-out objects.
	 */
	public void checkOut(IdType[] idTypes) throws ObjectIOException {
		throw new ObjectIOException("Operation not supported in this database");
	}
	/**
	 * get an object from the unique id. It doesn't reassemble the object, 
	* this should be provided by the handler.
	 */
	public StorableObject getObj(long id, Class objType)
		throws ObjectNotFoundException, ObjectIOException {
		UnpackedStorableObject objSpec = read(id, objType);
		if (objSpec == null) {
			org.eso.ohs.core.utilities.DebugUtils.stackTrace("SHOULDN'T HAPPEN");
			return null;
		}
		StorableObject sobj = (StorableObject)objSpec.getObject();
		return sobj;
	}
	/**
	 * get  object list from the where clause. It doesn't reassemble the objects, 
	 * this should be provided by the handler.
	 * After calling this method, you should register the objects to the object manager.
	 */
	public StorableObject[] getObj(
		String otherTables,
		String whereClause,
		Class objType)
		throws ObjectNotFoundException, ObjectIOException {
		UnpackedStorableObject[] objSpec =
			read(otherTables, whereClause, objType);
		if (objSpec == null) {
			DebugUtils.stackTrace("SHOULDN'T HAPPEN");
			return null;
		}
		StorableObject[] sobj = new StorableObject[objSpec.length];
		for (int i = 0; i < objSpec.length; i++) {
			sobj[i] = (StorableObject)objSpec[i].getObject();
		}
		return sobj;
	}
	/**
	 * updates the object
	 */
	public void updateObj(StorableObject obj, Class objType)
		throws ObjectNotFoundException, ObjectIOException {
		update(obj, objType);
	}
	
	/**
	* write the object
	*/
		public void writeObj(StorableObject obj, Class objType)
			throws ObjectNotFoundException, ObjectIOException {
			write(null, obj.getId(),objType,new UnpackedStorableObject(obj));
		}
			
		/**
		* delete the object
		*/
			public void deleteObj(StorableObject obj, Class objType)
				throws ObjectNotFoundException, ObjectIOException {
				delete(null, obj.getId(),objType);
			}
			
			/**
			* delete the object
			*/
				public void deleteWithObjCheck(StorableObject obj, Class objType)
					throws ObjectNotFoundException, ObjectIOException {
					delete(obj,objType);
				}		
				
		
	/**
	 * Returns a string representation of this object and its fields.
	 *
	 * @return a string representation.
	 */
	public String toString() {
		return "OpcDbaseMgr[rootNode=" + getRoot() + "]";
	}
	/**
	   * Return the one and only instance of the .
	   * 
	   * @return Singleton OpcDbaseMgr instance.
	   */
	public static OpcDbaseMgr getInstance() {
		if (singleton_ == null) {
			singleton_ = new OpcDbaseMgr();
		}
		return singleton_;
	}
	protected static class Phase1HandlerRegistry extends HandlerRegistry {
		/**
		 * Initialises the reistry with handler classes for every object type. 
		 * This method is called by the parent constructor. 
		 */
		protected void initialiseRegistry() {
			super.initialiseRegistry();
			stdlog_.debug("initialiseRegistry");
			register(new DbaseHandlerObsProgramme());
			register(new DbaseHandlerObsRun());
			register(new DbaseHandlerEsoUser());
			register(new DbaseHandlerOpcReferee());
			register(new DbaseHandlerTG1());
			register(new DbaseHandlerOR());
			register(new DbaseHandlerPanelObsRun());
			register(new DbaseHandlerCycle());
			register(new DbaseHandlerOpcPanel());
			register(new DbaseHandlerSchedRun());
			register(new DbaseHandlerOBTargetConstrainInfo());
            register(new DbaseHandlerTimeInterval());
            register(new DbaseHandlerRunProperties());
			register(new DbaseHandlerReportCard());
			register(new DbaseHandlerCommentCard());
            register(new DbaseHandlerProposalSummaryDBFields());
            register(new DbaseHandlerQueue());
		}
	}
    /* (non-Javadoc)
     * @see org.eso.ohs.persistence.StorageManager#checkOutReadme(org.eso.ohs.dfs.ObservingRun)
     */
    public String checkOutReadme(ObservingRun obsRun) throws ObjectIOException {
        throw new ObjectIOException("Object Not Implemented for this class");
    }
    /* (non-Javadoc)
     * @see org.eso.ohs.persistence.StorageManager#populateReadmeFile(org.eso.ohs.dfs.ObservingRun)
     */
    public Readme populateReadmeFile(ObservingRun obsRun) throws ObjectIOException {
        throw new ObjectIOException("Object not impelemented for this class");
    }
    /* (non-Javadoc)
     * @see org.eso.ohs.persistence.StorageManager#populateReadmeFileReplicate(org.eso.ohs.dfs.ObservingRun)
     */
    public Readme populateReadmeFileReplicate(ObservingRun obsRun) throws ObjectIOException {
        throw new ObjectIOException("Object not impelemented for this class");
    }
}
