package org.eso.ohs.persistence.dbase.phase2;

import java.net.UnknownHostException;
import java.sql.CallableStatement;
import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.Hashtable;

import org.apache.log4j.Logger;
import org.eso.ohs.core.utilities.ConnectionPool;
import org.eso.ohs.core.utilities.SqlUtils;
import org.eso.ohs.dfs.ObservationBlock;
import org.eso.ohs.persistence.Config;

/**
 * Class for logging OB events. Instances of this class are not created - all
 * calls are statis.
 * Events can only be added to the log.
 * They can never be deleted or changed by applications.
 * <p>
 * @version $Revision: 1.1 $
 * @author Ron J Lobo
 */
public class OBHistory {

    /** CVS version id */
    public static final String cvsId_ =
        "$Id: OBHistory.java,v 1.1 2005/03/30 09:56:12 tcanavan Exp $";

    /** Standard log channel */
    private static final Logger stdlog_ = Logger.getLogger(OBHistory.class);

    private static final String obrepDbName =
        Config.getCfg().getDbName(ObservationBlock.getSuffix());

    private static final String OE = obrepDbName + "..ob_events";
    private static final String OER = obrepDbName + "..ob_event_register";
    
    private static String dbURL;
    private static String dbUser;
    private static String dbPassword;

    private static Hashtable regHT = new Hashtable();

    private static String key_ob_event_id = "OBEVENTID";
    private static String key_ob_event_reg_id = "OBEVREGID";
    
    /**
     * The hostname where the application is running (initialized lazily).
     */
    private static String hostname;


    /**
     * constructor - unused
     */
    private OBHistory() {}


    /**
     * Method to add events to the log.
     * This method consist of 4 modifications to the database. The 4 steps are
     * not performed in a transaction as two of the steps (getting keys) can not
     * be included in a transaction.
     * The 4 steps, if interrupted, will not result in corrupt data, simply
     * that extra keys may be consumed or an entry is made in event_register 
     * for which there is not corresponding event. These are minor problems
     * and do not corrupt the user data.
     * The connection is set to setAutoCommit(true).
     * <br>
     * The parameters 'template_name', 'component' and 'login' may be null.
     * If 'login' is null, it will default to the user's account name.
     * <br>
     * @exception SQLException          an error occurred during the dbase operation
     * @exception UnknownHostException  the hostname couldn't be determined
     */
    public static void addEvent(long ob_id, String template_name,
        String component, String login, String msg)
    throws SQLException {

        Connection con = null;
        Statement stmt = null;
        try {

            if (msg == null) {
                throw new SQLException("Invalid null string: msg");
            }

            if (login == null) {
                login = System.getProperty("user.name");
            }

            if (hostname == null) {
                try {
                    hostname = java.net.InetAddress.getLocalHost().getHostName();
                } catch (UnknownHostException uhe) {
                    stdlog_.error("unknown hostname: " + uhe.getMessage());
                    hostname = "unknown";
                }
            }

            if (dbURL == null) {
                Config cfg = Config.getCfg();
                dbURL = cfg.getUrl(ObservationBlock.getSuffix());
                dbUser = cfg.getDbUserName();
                dbPassword = cfg.getDbPassword();
            }
            con = ConnectionPool.getPool().acquireConnection(
                dbURL, dbUser, dbPassword);

            // get key before the main transaction
            long ob_events_id = getKey(key_ob_event_id, con);

            con.setAutoCommit(true);
            con.createStatement();

            // check if register entry is already in local hashtable cache
            // add hash character between string tokens to avoid two keys being identical,
            // even if they have different components e.g. ""+"D"+"" is the same as "D"+""+"".
            String eventStr = ob_id+"#"+template_name+"#"+component+"#"+login+"#"+msg;
            Long oerId = (Long)regHT.get(eventStr);
            long ob_event_register_id;
            if (oerId != null) {
                ob_event_register_id = oerId.longValue();
                // stdlog_.debug("oerId found in cache = " + oerId);
            } else {
                // not found in cache - query DB to see if it exists
                ob_event_register_id = getRegisterId(con, template_name,
                    component, hostname, login, msg);
                regHT.put(eventStr, new Long(ob_event_register_id));
            }

            // add to OE table
            long event_time = (new java.util.Date()).getTime() / 1000;
            // stdlog_.debug("now=" + event_time + "=" + new java.util.Date(event_time * 1000));
            String sql = "insert into " + OE +
                " (ob_events_id, ob_event_register_id, ob_id, event_time)" +
                " values (" +
                ob_events_id + ", " +
                ob_event_register_id + ", " +
                ob_id + ", " +
                event_time + ")";
            stdlog_.info(sql);
            stmt = con.createStatement();
            stmt.executeUpdate(sql);
            stmt.close();

            stdlog_.debug("All OBHistory.addEvent() SQL commands executed.");
            con.commit();
            con.setAutoCommit(true);
            ConnectionPool.getPool().releaseConnection(con);

        } catch (SQLException e1) {
            stdlog_.error("OBHistory.addEvent: " + e1.getMessage());
            if (con != null) {
                try {
                    con.rollback();
                    stdlog_.debug("rollback done");
                } catch (SQLException e2) {
                    stdlog_.error("Error rolling back: " + e2.getMessage());
                }
                ConnectionPool.getPool().releaseConnection(con);
            }
            throw e1;
        }
    }


    /**
     * Get the event register id for the given event, or -1 if not
     * found. If add is true, add the event to the register (if not already there).
     * @param con
     * @param template_name
     * @param component
     * @param hostname
     * @param login
     * @param msg
     * @param add add the message if not present.
     * @return long
     * @throws SQLException
     */
    private static long getRegisterId(Connection con,
        String template_name, String component,
        String hostname, String login, String msg)
    throws SQLException {

        String templateNameCondition = template_name == null ? 
            " and template_name is null " :
            " and template_name=" + SqlUtils.sqlEscapedString(template_name);
        String componentCondition = component == null ?
            " and component is null " :
            " and component=" + SqlUtils.sqlEscapedString(component);
        String sql = "select ob_event_register_id from " + OER +
            " where hostname=" + SqlUtils.sqlEscapedString(hostname) +
            " and login=" + SqlUtils.sqlEscapedString(login) +
            " and msg=" + SqlUtils.sqlEscapedString(msg) +
            componentCondition +
            templateNameCondition;
        stdlog_.info(sql);

        Statement stmt = con.createStatement();
        // long start = (new java.util.Date()).getTime();
        ResultSet rs = stmt.executeQuery(sql);
        // long end = (new java.util.Date()).getTime();
        // stdlog_.debug("lookup in OER took: " + (end - start) + " millis");

        long ob_event_register_id = -1;
        if (rs.next()) {
            // event found in event register
            ob_event_register_id = rs.getLong("ob_event_register_id");
        }
        rs.close();
        stmt.close();
        if (ob_event_register_id == -1) {
            // event NOT found in event register - create entry
            ob_event_register_id = addToEventRegister(con,
                template_name, component, hostname, login, msg);
        }
        return ob_event_register_id;
    }


    /**
     * Add an event to the register, and return the id of the event.
     * @param con
     * @param template_name
     * @param component
     * @param hostname
     * @param login
     * @param msg
     * @return long
     * @throws SQLException
     */
    private static long addToEventRegister(Connection con,
        String template_name, String component,
        String hostname, String login, String msg)
    throws SQLException {
        long rc = getKey(key_ob_event_id, con);
        String t1, t2;
        Statement stmt = con.createStatement();
        if (template_name == null) {
            t1 = t2 = "";
        } else {
            t1 = "template_name, ";
            t2 = SqlUtils.sqlEscapedString(template_name) + ", ";
        }

        String c1, c2;
        if (component == null) {
            c1 = c2 = "";
        } else {
            c1 = "component, ";
            c2 = SqlUtils.sqlEscapedString(component) + ", ";
        }

        // add to event register table
        String sql = "insert into " + OER +
            " (ob_event_register_id, " + t1 + c1 + " hostname, login, msg)" +
            " values (" +
            rc + ", " +
            t2 +
            c2 +
            SqlUtils.sqlEscapedString(hostname) + ", " +
            SqlUtils.sqlEscapedString(login) + ", " +
            SqlUtils.sqlEscapedString(msg) + ")";
        stdlog_.info(sql);
        stmt.executeUpdate(sql);
        stmt.close();
        return rc;
    }

    /**
     * Get an id for the specified key. This is done through a call to 
     * a stored procedure.
     * @param key
     * @param con
     * @return long
     * @throws SQLException
     */
    private static long getKey(String key, Connection con) throws SQLException {

        // stdlog_.debug("calling stored proc opc_get_key, key=" + key);
        CallableStatement cstmt = con.prepareCall(
            "{? = call " + obrepDbName + "..opc_get_key(?)}");
        cstmt.registerOutParameter(1, java.sql.Types.INTEGER); // NUMERIC?
        cstmt.setString(2, key);
        // stdlog_.debug("C Stmt = " + cstmt);
        cstmt.execute();
        cstmt.getMoreResults();
        cstmt.getMoreResults();
        ResultSet rs = cstmt.getResultSet();
        if (rs != null) {
            rs.next();
            // long l = Config.getCfg().tableToUniqueId(rs.getLong(1), objType);
            long keyL = rs.getLong(1);
            stdlog_.debug("key: " + key + " = " + keyL);
            rs.close();
            cstmt.close();
            return keyL;
        }
        throw new SQLException("key not found");
    }
}

