package org.eso.ohs.phase2.actions;
import java.awt.Dimension;
import java.awt.event.ActionEvent;

import javax.swing.JComponent;
import javax.swing.JFrame;
import javax.swing.JOptionPane;

import org.eso.ohs.core.gui.baseaction.ActionSuperclass;
import org.eso.ohs.core.gui.baseaction.IncrementalAction;
import org.eso.ohs.core.gui.baseaction.IncrementalActionAutomaton;
import org.eso.ohs.core.gui.widgets.ErrorMessages;
import org.eso.ohs.core.gui.widgets.TextDisplayWidget;
import org.eso.ohs.core.utilities.ObjUtils;
import org.eso.ohs.core.utilities.ObjectIOException;
import org.eso.ohs.core.utilities.TextUtils;
import org.eso.ohs.dfs.BusinessObject;
import org.eso.ohs.dfs.CalibrationBlock;
import org.eso.ohs.evm.EvmExecutorImpl;
import org.eso.ohs.persistence.Config;
import org.eso.ohs.persistence.Media;
import org.eso.ohs.persistence.ObjectContainer;
import org.eso.ohs.persistence.ObjectManager;
import org.eso.ohs.persistence.ObjectNotFoundException;

/**
 * Action class to perform stage 2 (EVM) verification
 * on the selected BusinessObjects.
 * 
 *  0 OB(s) have verification errors.
 *  0 OB(s) verified succesfully. 
 *
 * @author Nick Kornweibel
 * <br>ChangeLog: cmavros 11-10-01 Add errorList to report
 * <br>ChangeLog: cmavros 2002/03/04 1690-OT, Display all evms messages 
 * <br>ChangeLog: cmavros 2002/JUNE/03 [1766/P2PP], Fail verification if OB name is blank.
 * @version $Id: Stage2VerifyBOAction.java,v 1.15 2005/11/09 15:42:07 ddorigo Exp $
 */

public class Stage2VerifyBOAction extends ActionSuperclass {

    /**
     * 
     */
    private static final long serialVersionUID = 1L;

    protected ObjectContainer folderView_;

    private Media device_;

    protected boolean folder_;

    protected StringBuffer msgLog = null;

    protected static TextDisplayWidget reportLog_ =
        new TextDisplayWidget("Verification Message Log");

    /**
     * numErrs and numGood give the number of BO's which have been
     * verified positifely or negatively
     */
    protected int numErrs = 0;
    protected int numGood = 0;

    /**
     * indicates whether the end of the iteration is reached or if we are
     * still processing the body of set of data upon which the iteration is
     * being performed
     */
    private boolean finished_ = false;

    public final static String SEPARATOR =
        "\n..................................................";

    /**
     * Constructs a VerifyBOAction action.
     */

    public Stage2VerifyBOAction(
        ObjectContainer folderView,
        boolean folder,
        String name) {
        super((JComponent) folderView, name);

        folderView_ = folderView;
        folder_ = folder;
    }

    /**
     * Event handler for the VerifyBOAction action.
     * 
     * @param ev       user-initiated ActionEvent to quit.
     * <BR><strong>ChangeLog</strong>: Toni Schlichting !! see document "item_0843.txt" accordingly
     */

    public void actionPerformedImpl(ActionEvent evt) {
        Long[] idLongs = null;

        numErrs = 0;
        numGood = 0;

   
        if (!folder_) {
            idLongs = folderView_.getAllSelected();
        } else {
            idLongs = folderView_.getAll();
        }

        idLongs = (Long[]) ObjUtils.uniqueOccurrences(idLongs);

        if (idLongs == null || idLongs.length == 0) {
            JOptionPane.showMessageDialog(
                folderView_.getTopLevelAncestor(),
                new Object[] {
                    "There are no selected Business Objects ",
                    "Which object should be verified?",
                    },
                "Nothing selected",
                JOptionPane.ERROR_MESSAGE);
            return; // nothing selected?
        }

        new IncrementalActionAutomaton(
            new IncrementalVerify(idLongs),
            (JFrame) folderView_.getTopLevelAncestor(),
            "Verifying ...");
    }

    public class IncrementalVerify implements IncrementalAction {

        private ObjectManager objMgr = ObjectManager.getObjectManager();
        private Long[] idLongs_;
        private Class objType_;

        public IncrementalVerify(Long[] ids) {
            idLongs_ = ids;
        }

        /**
         * @param  int counter - will be passed to the iteration method
         * @return number of steps remaining
         */
        public int getNumStepsRemaining(int counter) {
            int rc = 0;
            if (!finished_) {
                rc = idLongs_.length - counter;
            }
            return rc;
        }

        public void startOfAction() {
            finished_ = false;
            /** cmavros 11-10-01 Remove everything from errorList Vector 
             *  cmavros 19-10-01 Remove everything from errorListMsg Vector 
             *  clear all message bufffers and add header information so it is
             *  visible in all reports.
             */
            msgLog = new StringBuffer();
            reportLog_.clearTextArea();
            reportLog_.setVisible(false);
            setReportHeader();
        }

        public void endOfAction() {
            showFinalMsg();
        }

        /**
         * @param  int counter - will be passed to the iteration method
         */
        public void performStep(int counter) {
            if (!finished_) {
                doVerify(idLongs_[counter]);
            }
            if (counter + 1 == idLongs_.length) {
                finished_ = true;
            }
        }

        /**
         * doVerify does the verification for the BO identified with the
         *          parameter "id"
         * @param Long id
         */

        public void doVerify(Long id) {

            // TODO: refactor code so that the EVM verification code and
            // TODO: reporting is not duplicated between this action and
            // TODO: VerifyBOAction.java

            try {
                device_ = Config.getCfg().getMediaFromId(id.longValue());
                objType_ = Config.getCfg().getClassFromId(id.longValue());
                BusinessObject bo =
                    (BusinessObject) objMgr.getBusObj(
                        device_,
                        id.longValue(),
                        objType_);

                /** Get id number for the report */
                long idNum;
                if (device_.equals(Media.DBASE)) {
                    idNum = bo.getDbaseId();
                } else {
                    idNum =
                        (bo.getDbaseId() != 0)
                            ? Config.getCfg().uniqueToTableId(bo.getDbaseId())
                            : bo.getId();
                    if (idNum < 0)
                        idNum = 0;
                }

                String idNumber = "" + idNum;

                /** If the error is not already in the errorList 
                 *  append it for the report.
                 */

                msgLog.append("\n ");
                for (int i = 0; i < (10 - idNumber.length()); i++)
                    msgLog.append(" ");
                msgLog.append(idNum);
                for (int i = 0; i < (28 - bo.getName().length()); i++)
                    msgLog.append(" ");
                msgLog.append("" + bo.getName() + " : ");

                if (bo.getName() == null
                    || bo.getName().trim().length() == 0) {
                    msgLog.append("FAILED");
                    msgLog.append("\n");
                    msgLog.append("Name field cannot be blank");
                    msgLog.append(SEPARATOR);
                    return;
                }

                if (bo.verify().length() > 0) {
                    msgLog.append("FAILED");
                    msgLog.append(bo.verify());
                    msgLog.append(SEPARATOR);
                    return;
                }

                // Perform the Stage 2 verification
                String stage2VerifErrors = new String();
                StringBuffer header = new StringBuffer();
                StringBuffer footer = new StringBuffer();
                StringBuffer report = new StringBuffer();

                if (bo instanceof CalibrationBlock
                    && EvmExecutorImpl.evmConfigured()) {

                    boolean verifyPassed = true;
                    EvmExecutorImpl evm = EvmExecutorImpl.getInstance();

                    //Check that the evm is initialized
                    if (!evm.isInitialized()) {
                        stage2VerifErrors =
                            "\n Stage 2 Verification not performed, "
                                + "verification module failed to initialize: "
                                + evm.getStatusString();
                        verifyPassed = false;
                    } else {
                        //Initialization passed run verify
                        verifyPassed =
                            evm.verify(
                                (CalibrationBlock) bo,
                                report,
                                header,
                                footer);
                        //Format report message which includes 
                        //info, warnings, and errors for report 
                        // cmavros 2002/03/04 1690-OT
                        stage2VerifErrors =
                            "\n\n" + report.toString().trim() + "\n";
                    }

                    //Load report with messages
                    if (header.length() > 0) {
                        msgLog.append("\n" + header.toString().trim() + "\n");
                    }

                    if (!verifyPassed) {
                        msgLog.append("FAILED");
                        msgLog.append(stage2VerifErrors.toString());
                        numErrs++;
                    } else {
                        msgLog.append("OK");
                        msgLog.append(stage2VerifErrors.toString());
                        numGood++;
                    }

                    msgLog.append(SEPARATOR);

                    if (footer.length() > 0) {
                        msgLog.append(footer.toString());
                    }

                }
            } catch (ObjectNotFoundException e) {
                e.printStackTrace();
                ErrorMessages.announceNoObject(null, e);
                return;
            } catch (ObjectIOException e) {
                ErrorMessages.announceIOError(null, e);
                return;
            } //end try
        }

        /**
         * showFinalMsg shows a dialog box with information
         */

        public void showFinalMsg() {
            msgLog.append("\n");
            msgLog.append("\n\n " + numErrs + " OB(s) have verification errors.");
            msgLog.append("\n " + numGood + " OB(s) verified succesfully.");
                  reportLog_.setTextArea(msgLog.toString());
            reportLog_.setSize(new Dimension(700, 300));
            reportLog_.setVisible(true);
        }

        /**
         * Format the reprot Header
         */
        public void setReportHeader() {
            /** Report header */
            msgLog.append(
                "VERIFICATION MESSAGE LOG "
                    + TextUtils.currentISODateTime()
                    + "(UT)\n");
            msgLog.append(
                "==================================================\n\n");
            msgLog.append(
                "     ID" + "                      OB Name | Result\n");
            msgLog.append(
                "-------------" + "          ----------------------------");

        }
    }
}
