package org.eso.ohs.phase2.visiplot;

/**
 * VisiPlot.java
 *
 * Created on 1 August 2001 13:00 PM
 *
 * @version $Id: VisiPlot.java,v 1.24 2005/08/24 08:57:21 pnunes Exp $
 * @author cmavros  for ESO
 * <br>ChangeLog: cmavros 24 Apr 02 - 1722/OT 
 */

import java.util.Date;

import org.apache.log4j.Logger;
import org.eso.ohs.core.utilities.Convert;
import org.eso.ohs.dfs.ConstraintSet;
import org.eso.ohs.dfs.ObservationBlock;
import org.eso.ohs.dfs.Target;
import org.eso.ohs.dfs.TimeInterval;
import org.eso.ohs.evm.EvmExecutorImpl;
import org.eso.ohs.persistence.AppConfig;
import org.eso.ohs.phase2.visibility.AirMass;
import org.eso.ohs.phase2.visibility.AirMassFilter;
import org.eso.ohs.phase2.visibility.FLI;
import org.eso.ohs.phase2.visibility.FLIFilter;
import org.eso.ohs.phase2.visibility.JulianDate;
import org.eso.ohs.phase2.visibility.LST;
import org.eso.ohs.phase2.visibility.MoonDistance;
import org.eso.ohs.phase2.visibility.MoonDistanceFilter;
import org.eso.ohs.phase2.visibility.MoonDownFilter;
import org.eso.ohs.phase2.visibility.PCF;
import org.eso.ohs.phase2.visibility.PCFInterval;
import org.eso.ohs.phase2.visibility.PCFPlotter;
import org.eso.ohs.phase2.visibility.PCFPlotterFilter;
import org.eso.ohs.phase2.visibility.PCFPoint;
import org.eso.ohs.phase2.visibility.SiderealTime;
import org.eso.ohs.phase2.visibility.SunAngle;
import org.eso.ohs.phase2.visibility.SunAngleFilter;

public class VisiPlot {

    /** Standard log channel 
     * <br>ChangeLog: cmavros 21-12-01 added
     */
    private static Logger stdlog_ = Logger.getLogger(VisiPlot.class);

    private ConstraintSet cs_;

    /**
     * Constraint values
     */
    private double airConstraint_;
    private double fliConstraint_;
    private double moonConstraint_;
    private double sunConstraint_;
    private double timeIntervalConstraint_;
    private double siderelTimeIntervalConstraint_;

    private Date startTime_;
    private Date endTime_;
    private Date intervalTime_;
    private double execTime_;
    private int interval_;

    /**
     * VisiPlot Options 
     */
    private boolean airEnabled_;
    private boolean fliEnabled_;
    private boolean moonEnabled_;
    private boolean sunEnabled_;
    private boolean execTimeEnabled_;
    private boolean timeIntervalEnabled_;
    private boolean siderealTimeIntervalEnabled_;

    /**
     * Do not care about moon angle or FLI if the moon is not up
     * this option indicates if the moon up/down status should be 
     * considered or not (currently set to true, but maybe a configuration
     * option later).
     */
    private boolean moonVisibilityEnabled_ = true;

    /**
     * PCF Plotter values
     */
    private PCFPlotter fliPlotter_;
    private PCFPlotter moonDistPlotter_;
    private PCFPlotter airMassPlotter_;
    private PCFPlotter sunPlotter_;
    private PCFPlotter moonDownPlotter_;

    /**
     * PCFs 
     */
    private PCF resultPCF_;
    private PCF timeIntervalPCF_;
    private PCF siderealTimeIntervalPCF_;
    private PCF execTimePCF_;

    /**
     * Constant to convert milliarc seconds to  radians
     */
    private static final double MILLARC_HRMMSS = 7.272205216643E-8;
    private static final double MILLARC_MMSS = 206265.0;
    public static final int MILLISECS_PER_SEC = 1000;

    private static final int DURATION = 12;

    // PARANAl coordinates lat -24 37 33.117 
    // PARANAl coordinates long 70 24 11.642
    // http://www.eso.org/paranal/site/paranal.html
    // SLALIB should use the East +ve system but it seems to give incorrect
    // results - so I changed it back to West +ve.
    private double PLONG = 1.228768; // Radians 
    private double PLAT = -0.429802; // Radians
    private static final double PELEVSEA = 2635.43; //meters

    // LASILLA coordinates Latitude 29 15' south 
    // LASILLA coordinates Longitude 70 44' west 
    // http://www.ls.eso.org/lasilla/site/aboutls.html
    //private static double LLONG = 4.7153; //degress
    //private static double LLAT = -29.257; //degress
    //private static final double LELEVSEA = 2347.; //meters

    /**
     * Create and calculate PCFs.
     *
     * @param current ObservationBlock
     * @param current VisiPlot Options 
     */
    public VisiPlot(ObservationBlock ob_, VisiPlotOptions vpo_) {
        PLONG = AppConfig.getAppConfig().getLongitude();
        PLAT = AppConfig.getAppConfig().getLatitude();

        readVisiPlotOptions(ob_, vpo_);
        initAirMassPCF();
        calcSelectedPCF(ob_);
        //invertAirMass();  NOT USED AS OF 26/09/01
    }

    /**
     * <br>ChangeLog: cmavros 24 Apr 02  - 1722/OT
     * @return time interval pcf
     */
    public PCF getTimeIntervalPCF() {
        return (timeIntervalPCF_);
    }

    /**
     * <br>ChangeLog: cmavros 24 Apr 02  - 1722/OT
     * @return time interval pcf
     */
    public PCF getSiderealTimeIntervalPCF() {
        return (siderealTimeIntervalPCF_);
    }

    /**
     * @return exec time pcf
     */
    public PCF getExecTimePCF() {
        return (execTimePCF_);
    }

    /**
     * @return fli pcf
     */
    public PCF getFLIPCF() {
        return (fliPlotter_);
    }

    /**
     * @return moon dist pcf
     */
    public PCF getMoonDistPCF() {
        return (moonDistPlotter_);
    }

    /**
     * @return PCF of moon down status (1.0: its down, 0.0: its up).
     */
    public PCF getMoonDownPCF() {
        return (moonDownPlotter_);
    }

    /**
     * @return the start time of the plot
     */
    public Date getStartTime() {
        return startTime_;
    }

    /**
     * @return the end time of the plot 
     */
    public Date getEndTime() {
        return endTime_;
    }

    /**
     * @return the precison of the plot
     */
    public int getPrecision() {
        return interval_;
    }

    /**
     * @return true if AirMass is enabled 
     */
    public boolean isAirMassEnabled() {
        return airEnabled_;
    }

    /**
     * @return air mass pcf
     */
    public PCF getAirMassPCF() {
        return (airMassPlotter_);
    }

    /**
     * @return sun pcf
     */
    public PCF getSunPCF() {
        return (sunPlotter_);
    }

    /**
     * @return time interval 
     */
    public double getTimeInterval(ObservationBlock ob, double jdate) {
        return (getTimeIntervalValue(ob, jdate));
    }

    /**
     * Combine all of the selected PCFs
     * @return result pcf
     */
    public PCF getVisibilityPCF() {
        resultPCF_ = (PCF) airMassPlotter_.clone();

        if (fliEnabled_) {
            resultPCF_.confine(fliPlotter_, AirMass.NOT_VIS);
        }
        if (moonEnabled_) {
            resultPCF_.confine(moonDistPlotter_, AirMass.NOT_VIS);
        }
        if (sunEnabled_) {
            resultPCF_.confine(sunPlotter_, AirMass.NOT_VIS);
        }
        if (timeIntervalEnabled_) {
            resultPCF_.confine(timeIntervalPCF_, AirMass.NOT_VIS);
        }
        if (siderealTimeIntervalEnabled_) {
            resultPCF_.confine(siderealTimeIntervalPCF_, AirMass.NOT_VIS);
        }

        if (execTimeEnabled_) {
            createExecTimePCF();
        }
        //stdlog_.info(
        //    "\nvisiplot.Visiplot.getVisibilityPCF resultPCF FINAL:\n"
        //    +resultPCF_.toString());
        return (resultPCF_);
    }

    /**
     * readVisiPlotOptions gets the user selected options
     * from the VisiPlot selection options.
     * 
     * @param current ObservationBlock
     * @param current VisiPlot Options 
     */
    private void readVisiPlotOptions(
        ObservationBlock ob,
        VisiPlotOptions vpo) {
        String val = null;

        interval_ = vpo.getIntValue(VisiPlotOptions.dataQueryIntKey);
        airEnabled_ = vpo.isEnabled(VisiPlotOptions.airMassConEnabledKey);
        endTime_ = vpo.getISODate(VisiPlotOptions.endTimeKey);
        execTimeEnabled_ = vpo.isEnabled(VisiPlotOptions.obexecConKey);
        fliEnabled_ = vpo.isEnabled(VisiPlotOptions.fliConEnabledKey);
        moonEnabled_ = vpo.isEnabled(VisiPlotOptions.moonDistConEnabledKey);
        startTime_ = vpo.getISODate(VisiPlotOptions.startTimeKey);
        sunEnabled_ = vpo.isEnabled(VisiPlotOptions.sunConEnabledKey);
        timeIntervalEnabled_ =
            vpo.isEnabled(VisiPlotOptions.obtimeIntervalConKey);

        siderealTimeIntervalEnabled_ =
            vpo.isEnabled(VisiPlotOptions.obSiderealTimeIntervalConKey);

        val = vpo.getValue(VisiPlotOptions.airMassConKey);
        airConstraint_ = getAirMassConstraint(ob, vpo, val);

        val = vpo.getValue(VisiPlotOptions.moonDistConKey);
        moonConstraint_ = getMoonConstraint(ob, vpo, val);

        val = vpo.getValue(VisiPlotOptions.fliConKey);
        fliConstraint_ = getFLIConstraint(ob, vpo, val);

        val = vpo.getValue(VisiPlotOptions.sunConKey);
        sunConstraint_ = getSunConstraint(vpo, val);

        try {
            if (EvmExecutorImpl.evmConfigured()) {
                EvmExecutorImpl evm = EvmExecutorImpl.getInstance();
                //Check that the evm is initialized
                if (evm.isInitialized()) {
                    evm.calculateETCS(ob);
                }
            }
            execTime_ = ob.getExecutionTime();
        } catch (Exception e) {
            execTime_ = 0.;
        }
        /*
        stdlog_.info("\nvisiplot.Visiplot.readVisiPlotOptions Constraint if enabled: ");
        stdlog_.info("visiplot.Visiplot.readVisiPlotOptions airConstraint = " + airConstraint_);
        stdlog_.info("visiplot.Visiplot.readVisiPlotOptions moonConstraint = " + moonConstraint_);
        stdlog_.info("visiplot.Visiplot.readVisiPlotOptions fliConstraint = " + fliConstraint_);
        stdlog_.info("visiplot.Visiplot.readVisiPlotOptions sunConstraint = " + sunConstraint_);
        */

    }

    /**
     * Initialize the AirMass PCF
     */
    private void initAirMassPCF() {
        double[] values = new double[6];
        double step = 1.0;
        for (int i = 0; i < values.length; i++) {
            values[i] = step;
            step = step + 0.5;
        }
        AirMassFilter airMassFilter =
            new AirMassFilter(new double[0], airConstraint_);
        airMassPlotter_ = new PCFPlotter(AirMass.NOT_VIS, airMassFilter);
    }

    /**
     * Calculate the duration of the plot.
     * Subtract the received end date/time from the start date/time
     * divide the difference by the number of milliseconds in a day.  
     * Determine if the this is more or less than one day and 
     * return the duration.
     *
     * @param startDate Date
     * @param endDate Date
     * @return duration integer
     */
    private int getDuration(Date startDate, Date endDate) {
        double timeBetween = (double) (endDate.getTime() - startDate.getTime());
        timeBetween = timeBetween / (1000. * 60. * 60. * 24.);
        if (timeBetween < 1.)
            return (int) (timeBetween * 24.);
        else {
            return (int)
                ((Math.floor(timeBetween) * 24.)
                    + ((Math.floor(timeBetween) - timeBetween) * 24.));
        }
    }

    /**
     * calcSelectedPCF - Calculate the selected PCFs for the current target
     * for a specific time period. Loop thru the selected constraints
     * and pass the calculated value and time to the PCF.
     * @param ob current ObservationBlock 
     */
    private void calcSelectedPCF(ObservationBlock ob) {
        Target tg = ob.getTarget();
        double jDate = 0.0;
        double targetRA = 0.0;
        double targetDEC = 0.0;
        double sunAngle = 0.0;
        double fli = 0.0;
        double moonDist = 0.0;
        double moonUp = 0.0; // 1.0 if up
        double airMass = 0.0;
        double timeInterval = 0.0;
        int total = (60 / interval_) * getDuration(startTime_, endTime_);
        if (total <= 0) {
            total = (60 / interval_) * DURATION;
            stdlog_.info(
                "\nvisiplot.Visiplot.calcSelectedPCF Problem "
                    + "calculating duration of plot setting to 12");
        }
        //stdlog_.info("\nvisiplot.Visiplot.calcSelectedPCF interval = " +
        //  interval_ + " total = " + total);

        // Get Observatory lat, long from OB
        double observLong = PLONG;
        double observLat = PLAT;

        // Get target ra and dec from OB
        targetRA = tg.getRA();
        targetDEC = tg.getDeclination();

        // Convert from milliarc to radians
        targetRA = targetRA * MILLARC_HRMMSS;
        targetDEC = (targetDEC / MILLARC_MMSS) / MILLISECS_PER_SEC;

        /*
        stdlog_.info("visiplot.Visiplot.calcSelectedPCF Target RA " + 
          "Value  = " + targetRA + " radians");
        stdlog_.info("visiplot.Visiplot.calcSelectedPCF Target DEC " + 
          "Value =  " + targetDEC + " radians");
        stdlog_.info("\nvisiplot.Visiplot.calcSelectedPCF " + startTime_ + " UT");
         */

        /** 
         * To initialize the PCFs a start and end time is needed.
         * For the first iteration the intervalTime_ is set to
         * the  startTime_. The following iterations then
         * add the interval time to the intervalTime_.
         */
        intervalTime_ = startTime_;

        for (int i = 0; i < total; i++) {
            // Calculate the Julian Date plus the interval time
            jDate = getJulianDate(startTime_, interval_, i);
            // pass down millseconds. 
            Date utDate =
                new Date(startTime_.getTime() + (interval_ * i * 60000));
            stdlog_.info(
                "visiplot.Visiplot.calcSelectedPCF Julian Date: " + jDate);
            //Calculate the local sidereal time of the Observatory 
            double lst = getLST(jDate, observLong);

            //Calculate AirMass
            airMass =
                getAirMassValue(jDate, lst, targetRA, targetDEC, observLat);
            airMass = Convert.round(airMass, 6);
            stdlog_.info("    AirMass: " + airMass);

            if (i == 0) {
                airMassPlotter_.addInterval(
                    startTime_,
                    getIntervalEndTime(startTime_, interval_),
                    airMass);
            } else {
                airMassPlotter_.appendInterval(
                    getIntervalEndTime(intervalTime_, interval_),
                    airMass);
            }

            //Calculate FLI
            if (fliEnabled_) {
                fli = getFLIValue(jDate, observLong, observLat);
                //If this is the first time thru the loop create a new PCF
                if (i == 0) {
                    FLIFilter fliFilter = new FLIFilter(fliConstraint_);
                    fliPlotter_ = new PCFPlotter(1., fliFilter);
                    fliPlotter_.addInterval(
                        startTime_,
                        getIntervalEndTime(startTime_, interval_),
                        fli);
                }
                //Append to the existing PCF
                else
                    fliPlotter_.appendInterval(
                        getIntervalEndTime(intervalTime_, interval_),
                        fli);
                stdlog_.info("    FLI: " + fli);
            }

            // Calculate if the moon is up or not
            if (moonVisibilityEnabled_) {
                moonUp = getMoonVisible(jDate, observLong, observLat);
                if (i == 0) {
                    MoonDownFilter moonDownFilter_ = new MoonDownFilter();
                    moonDownPlotter_ = new PCFPlotter(0.0, moonDownFilter_);
                    moonDownPlotter_.addInterval(
                        startTime_,
                        getIntervalEndTime(startTime_, interval_),
                        moonUp);
                } else {
                    moonDownPlotter_.appendInterval(
                        getIntervalEndTime(intervalTime_, interval_),
                        moonUp);
                }
            }

            //Calculate moon angular distance
            if (moonEnabled_) {
                moonDist =
                    getMoonDistValue(
                        jDate,
                        targetRA,
                        targetDEC,
                        observLong,
                        observLat);
                if (i == 0) {
                    //If this is the first time thru the loop create a new PCF
                    MoonDistanceFilter moonDistFilter_ =
                        new MoonDistanceFilter(moonConstraint_);
                    moonDistPlotter_ = new PCFPlotter(0.0, moonDistFilter_);
                    moonDistPlotter_.addInterval(
                        startTime_,
                        getIntervalEndTime(startTime_, interval_),
                        moonDist);
                }
                //Append to the existing PCF
                else
                    moonDistPlotter_.appendInterval(
                        getIntervalEndTime(intervalTime_, interval_),
                        moonDist);
                stdlog_.info("    Moon: " + moonDist);
            }

            //Calculate sun angle 
            if (sunEnabled_) {
                sunAngle = getSunValue(jDate, observLong, observLat, lst);
                if (i == 0) {
                    //If this is the first time thru the loop create a new PCF
                    SunAngleFilter sunFilter =
                        new SunAngleFilter(sunConstraint_);
                    sunPlotter_ = new PCFPlotter(1., sunFilter);
                    sunPlotter_.addInterval(
                        startTime_,
                        getIntervalEndTime(startTime_, interval_),
                        sunAngle);
                }
                //Append to the existing PCF
                else
                    sunPlotter_.appendInterval(
                        getIntervalEndTime(intervalTime_, interval_),
                        sunAngle);
                stdlog_.info("    Sun: " + sunAngle);
            }

            //Calculate Time Intervals PCF
            if (timeIntervalEnabled_) {
                timeInterval = getTimeIntervalValue(ob, jDate);
                if (i == 0) {
                    //If this is the first time thru the loop create a new PCF
                    timeIntervalPCF_ = new PCF(0.0);
                    timeIntervalPCF_.addInterval(
                        startTime_,
                        getIntervalEndTime(startTime_, interval_),
                        timeInterval);
                }
                //Append to the existing PCF
                else
                    timeIntervalPCF_.appendInterval(
                        getIntervalEndTime(intervalTime_, interval_),
                        timeInterval);
                stdlog_.info("    Time Interval: " + timeInterval);
            }
            stdlog_.info(
                " Sidereal Time Interval : " + siderealTimeIntervalEnabled_);
            if (siderealTimeIntervalEnabled_) {
                timeInterval = getSiderealTimeIntervalValue(ob, utDate, jDate);
                if (i == 0) {
                //If this is the first time thru the loop create a new PCF
                siderealTimeIntervalPCF_ = new PCF(0.0);
                siderealTimeIntervalPCF_.addInterval(
                    startTime_,
                    getIntervalEndTime(startTime_, interval_),
                    timeInterval);
                }
                //Append to the existing PCF
                else
                    siderealTimeIntervalPCF_.appendInterval(
                    getIntervalEndTime(intervalTime_, interval_),
                    timeInterval);
                stdlog_.info("    Sidereal Time Interval: " + timeInterval);
            }
            //Get next interval intervalTime_
            intervalTime_ = getIntervalEndTime(intervalTime_, interval_);
        }

        // if the moon up/down status is considered, the moonDownPlotter_
        // must be added to both the fliPlotter_ and moonDistPlotter_
        // (if they are enabled).
        if (moonVisibilityEnabled_) {
            if (fliEnabled_) {
                PCFPlotterFilter flt = fliPlotter_.getFilter();
                fliPlotter_.setFilter(null);
                fliPlotter_.add(moonDownPlotter_, 1.0);
                fliPlotter_.setFilter(flt);
            }
            if (moonEnabled_) {
                PCFPlotterFilter flt = moonDistPlotter_.getFilter();
                moonDistPlotter_.setFilter(null);
                moonDistPlotter_.add(moonDownPlotter_, 1.0);
                moonDistPlotter_.setFilter(flt);
            }
        }
    }

    /**
     * invertAirMass set the y-axis of the plot
     * NOT USED AS OF 26/09/01
     *
     */
    private void invertAirMass() {
        int pcfLength = airMassPlotter_.getIntervalCount();
        for (int i = 0; i < pcfLength; i++) {
            PCFPoint p = airMassPlotter_.getPointAt(i);
            if (p.getValue() > 3.5)
                p.setValue(0.0);
            if (p.getValue() == 3.5)
                p.setValue(0.0);
            if (p.getValue() == 3.)
                p.setValue(0.2);
            if (p.getValue() == 2.5)
                p.setValue(0.4);
            if (p.getValue() == 2.)
                p.setValue(0.6);
            if (p.getValue() == 1.5)
                p.setValue(0.8);
            if (p.getValue() == 1.)
                p.setValue(1.0);
        }
        //stdlog_.info("\nvisiplot.Visiplot.invertAirMass AirMassPCF:\n"+airMassPlotter_.toString());
    }

    /**
     * Create a execTimePCF that is a clone of the resultPCF.
     * Loop thru the execTimePCF to the time interval  
     * that is less than the AirMass.NOT_VIS value.
     * Subtract the exec time from this interval.  This
     * is the start of the exec time.
     * Set this execTimePCF interval to 1 and all other intervals in
     * the execTimePCF to 0. 
     *
     */
    private void createExecTimePCF() {
        int ctr = 0;
        execTimePCF_ = (PCF) resultPCF_.clone();
        int pcfLength = execTimePCF_.getIntervalCount();
        //Get last entry with a value that is < AirMass.NOT_VIS
        for (int i = 0; i < pcfLength; i++) {
            PCFPoint p = execTimePCF_.getPointAt(i);
            if ((p.getValue() < AirMass.NOT_VIS) && (p.getValue() > 0.)) {
                ctr = i;
            }
            p.setValue(0.);
        }
        if (ctr < pcfLength) {
            PCFInterval rp = execTimePCF_.getIntervalAt(ctr);
            if (execTime_ > 0.) {
                int xTime = (int) (Math.floor(execTime_ / 60.));
                if (xTime > 0) {
                    Date jDate = JulianDate.getJDC(rp.getEnd(), -xTime);
                    execTimePCF_.setInterval(jDate, rp.getEnd(), 1.);
                }
            }
        }
        //stdlog_.info("\nvisiplot.Visiplot.createExecTimePCF execTimePCF:\n"+execTimePCF_.toString() );
    }

    /**
     * Remove OB execution time(secs) from the result PCF.
     * If the execTime of the OB is > than 0 subtract
     * the execution Time from the date of the last PCF interval
     * that less than AirMass.NOT_VIS.
     * 
     * NOT USED AS OF 01/10/01
     *
     */
    private void subtractExecTime() {
        int ctr = 0;
        int pcfLength = resultPCF_.getIntervalCount();
        //Get last entry with a value that is < AirMass.NOT_VIS
        for (int i = 0; i < pcfLength; i++) {
            PCFPoint p = resultPCF_.getPointAt(i);
            if ((p.getValue() < AirMass.NOT_VIS) && (p.getValue() > 0)) {
                ctr = i;
            }
        }
        if (ctr < pcfLength) {
            PCFInterval rp = resultPCF_.getIntervalAt(ctr);
            if (execTime_ > 0.) {
                int xTime = (int) (Math.floor(execTime_ / 60.));
                if (xTime > 0) {
                    Date jDate = JulianDate.getJDC(rp.getEnd(), -xTime);
                    resultPCF_.setInterval(jDate, rp.getEnd(), AirMass.NOT_VIS);
                }
            }
        }
        //stdlog_.info("\nvisiplot.Visiplot.subtractExecTime resultPCF:\n"+resultPCF_.toString());
    }

    /**
     * getJulianDate get Julian date + interval
     * @param startdate Date
     * @param interval integer time of each plot interval
     * @param ctr integer the position the loop
     * @return Julian date double 
     * see JulianDate#getJDC(Date, int, int)
     */
    private double getJulianDate(Date startDate, int interval, int ctr) {
        double jdate = JulianDate.getJDC(startDate, interval, ctr);
        return (jdate);
    }

    /**
     * getIntervalEndTime get start date + interval
     * @param startdate Date 
     * @param interval integer
     * see JulianDate#getJDC(Date, int)
     */
    private Date getIntervalEndTime(Date startDate, int interval) {
        Date jdate = JulianDate.getJDC(startDate, interval);
        return (jdate);
    }

    /**
     * getDateZeroHours get Julian date at 0 hours 0 minutes and 0 secs
     * @param startdate Date 
     * @return Julian date double 
     * see JulianDate#getJDC(Date)
     */
    private double getDateZeroHours(Date startDate) {
        double jdate = JulianDate.getJDC(startDate);
        return (jdate);
    }

    /**
     * getLST get the local side real time
     * Substract the longitude of the Observatory from
     * the returned sidereal time for the local sidereal time
     * of the observatory 
     * @param jdate double Julian Date
     * @param longit double the longitude of the Observatory
     * @return lst double 
     * @see SiderealTime#getST(double)
     */
    private double getLST(double jdate, double longitude) {
        double stValue = SiderealTime.getST(jdate);
        double lstValue = stValue - longitude;
        return (lstValue);
    }

    /**
     * getFLIValue - get the Fractional Lunar Illumination. 
     * @param jdate double Julian Date
     * @param longit double the longitude of the Observatory
     * @param lat double the latitude of the Observatory
     * @return FLI double 
     * @see FLI#getFLI(double, double, double, boolean)
     */
    private double getFLIValue(double jdate, double longit, double lat) {
        double fliValue = FLI.getFLI(jdate, longit, lat, true);
        return (fliValue);
    }

    /**
     * getAirMass of the target to the Observatory
     * @param jdate double Julian Date
     * @param lst double the Local Sidereal Time
     * @param target Ra double
     * @param target Dec double
     * @param lat double the latitude of the Observatory
     * @return Air Mass double
     * @see AirMass#calculateAirMass(double, double, double, double, double)
     */
    private double getAirMassValue(
        double jdate,
        double lst,
        double targetRA,
        double targetDEC,
        double lat) {
        double airMassValue =
            AirMass.calculateAirMass(jdate, lst, targetRA, targetDEC, lat);
        return (airMassValue);
    }

    /**
     * getMoonDistValue - get the Moon angle of seperation
     * from the target
     * @param jdate double Julian Date
     * @param target Ra double
     * @param target Dec double
     * @param longit double the longitude of the Observatory
     * @param lat double the latitude of the Observatory
     * @return Moon Distance
     * @see MoonDistance#getMoonDistance(double, double, double, double, double)
     */
    private double getMoonDistValue(
        double jdate,
        double targetRA,
        double targetDEC,
        double longit,
        double lat) {
        double moonValue =
            MoonDistance.getMoonDistance(
                jdate,
                targetRA,
                targetDEC,
                longit,
                lat);
        return (moonValue);
    }

    private double getMoonVisible(double jdate, double longit, double lat) {
        double moonValue = MoonDistance.getMoonVisible(jdate, longit, lat);
        return (moonValue);
    }

    /**
     * getSunValue - get the Sun Twilight angle 
     * @param jdate double Julian Date
     * @param longit double the longitude of the Observatory
     * @param lat double the latitude of the Observatory
     * @param lst double the Local Sidereal Time
     * @return sun angle in degrees
     * @see SunAngle#getSunAngle(double, double, double, double)
     */
    private double getSunValue(
        double jdate,
        double longit,
        double lat,
        double lst) {
        double saValue = SunAngle.getSunAngle(jdate, longit, lat, lst);
        return (saValue);
    }

    /**
     * getTimeIntervalValue - get the Time Interval Value 
     * If the time interval has not been defined in the OB
     * any time period is satisfactory set the returned value to 1. 
     * Else if the start and end time does not fall within
     * this interval time (precision) set returned value to 0. 
     * @param ObservationBlock
     * @param julian Date double
     * @return time interval value
     */
    private double getTimeIntervalValue(ObservationBlock ob, double jdate) {
        TimeInterval[] timeInts = ob.getTimeIntervals();
        double timeValue = 1.;

        if (timeInts != null && timeInts.length != 0) {
            for (int i = 0; i < timeInts.length; i++) {
                Date startTime =
                    new Date(timeInts[i].getStartTime() * MILLISECS_PER_SEC);
                Date endTime =
                    new Date(timeInts[i].getEndTime() * MILLISECS_PER_SEC);
                // Convert start and end time to Julian Date 
                double jstart = JulianDate.getJDC(startTime, 0, 0);
                double jend = JulianDate.getJDC(endTime, 0, 0);
                /*
                stdlog_.info("visiplot.VisiPlot.getTimeIntervalValue start: "+jstart); 
                stdlog_.info("visiplot.VisiPlot.getTimeIntervalValue   end: "+jend);
                stdlog_.info("visiplot.VisiPlot.getTimeIntervalValue jdate: "+jdate); 
                */
                if ((jdate >= jstart) && (jdate <= jend)) {
                    timeValue = 1.;
                    return (timeValue);
                } else
                    timeValue = 0.;
            }
        }
        return (timeValue);
    }


    /**
     * getTimeIntervalValue - get the Sidereal Time Interval Value If the time
     * interval has not been defined in the OB any time period is satisfactory
     * set the returned value to 1. Else if the start and end time does not fall
     * within this interval time (precision) set returned value to 0.
     * @param ObservationBlock
     * @param julian Date double
     * @return time interval value
     */
    private double getSiderealTimeIntervalValue(
        ObservationBlock ob,
        Date utDate,
        double jdate) {

        // Convet the jdate to LST in radioans.

        //Convert user values into radian
        //Start and end dates for ST Times.
        // If the end date is < start date then 
        // add 2 PI to end date. 

        // Then compate the values         

        TimeInterval[] timeInts = ob.getSTTimeIntervals();
        double timeValue = 1.;
        LST lst = new LST();

        double siderealTimeInRadians = lst.getLST(jdate);
        siderealTimeInRadians = lst.getLST(jdate) - PLONG;
        if (siderealTimeInRadians < 0) {
            siderealTimeInRadians = siderealTimeInRadians + (2 * Math.PI);
        }
        if (siderealTimeInRadians > (2 * Math.PI)) {
            siderealTimeInRadians = siderealTimeInRadians - (2 * Math.PI);
        }
        if (timeInts != null && timeInts.length != 0) {
            for (int i = 0; i < timeInts.length; i++) {
                double startTime =
                    timeInts[i].getStartTime() * (Math.PI / (12 * 3600));
                double endTime = timeInts[i].getEndTime() * (Math.PI / (12 * 3600));
                if (((siderealTimeInRadians >= startTime)
                    && (siderealTimeInRadians <= endTime))
                       || ((siderealTimeInRadians + (2 * Math.PI) >= startTime)
                        && (siderealTimeInRadians + (2 * Math.PI) <= endTime))) {
                    timeValue = 1.;
                    return (timeValue);
                } else {
                    timeValue = 0.;

                }
            }
        }
        return (timeValue);
    }

    /**
     * getAirMassConstraint calls isReadFromOB()
     * to determines which constraint value to use; 
     * the VisiPlot Options value or the constraint 
     * from the database.
     *
     * @return AirMass Constraint 
     * @see #readVisiPlotOptions(ObservationBlock ob, VisiPlotOptions vpo)
     * @see VisiPlotOptions#isReadFromOB(String value)
     */
    private double getAirMassConstraint(
        ObservationBlock ob,
        VisiPlotOptions vpo,
        String value) {
        airConstraint_ = 0.;
        cs_ = ob.getConstraintSet();
        if (airEnabled_) {
            boolean readFromOB = VisiPlotOptions.isReadFromOB(value);
            if (!readFromOB) {
                try {
                    airConstraint_ = Double.valueOf(value).doubleValue();
                } catch (NumberFormatException e) {
                    airConstraint_ = 0.;
                }
            } else {
                airConstraint_ = cs_.getAirmass();
            }
        }
        //stdlog_.info("visiplot.Visiplot.getAirMassConstraint " + airConstraint_);  
        return (airConstraint_);
    }

    /**
     * getFLIConstraint calls isReadFromOB()
     * to determines which constraint value to use; 
     * the VisiPlot Options value or the constraint 
     * from the database.
     *
     * @return FLI Constraint 
     * @see #readVisiPlotOptions(ObservationBlock ob, VisiPlotOptions vpo)
     * @see VisiPlotOptions#isReadFromOB(String value)
     */
    private double getFLIConstraint(
        ObservationBlock ob,
        VisiPlotOptions vpo,
        String value) {
        cs_ = ob.getConstraintSet();
        boolean readFromOB = VisiPlotOptions.isReadFromOB(value);
        if (!readFromOB) {
            try {
                fliConstraint_ = Double.valueOf(value).doubleValue();
            } catch (NumberFormatException e) {
                fliConstraint_ = 0.;
            }
        } else {
            fliConstraint_ = cs_.getFractionalLunarIllumination();
        }
        //System.out.print(" visiplot.Visiplot.getFLIConstraint " + fliConstraint_);  
        return (fliConstraint_);
    }

    /**
     * getSunConstraint calls isReadFromOB()
     * to determines if the default value of -18 or
     * the VisiPlot Options value is used
     *
     * @return Sun Constraint
     * @see #readVisiPlotOptions(ObservationBlock ob, VisiPlotOptions vpo)
     * @see VisiPlotOptions#isReadFromOB(String value)
     */
    private double getSunConstraint(VisiPlotOptions vpo, String value) {
        boolean readFromOB = VisiPlotOptions.isReadFromOB(value);
        if (!readFromOB) {
            try {
                sunConstraint_ = Double.valueOf(value).doubleValue();
            } catch (NumberFormatException e) {
                sunConstraint_ = -18.;
            }
        } else {
            sunConstraint_ = -18.;
        }
        //System.out.print(" visiplot.Visiplot.getSunConstraint " + sunConstraint_);
        return (sunConstraint_);
    }

    /**
     * getMoonDistance calls isReadFromOB()
     * to determines which constraint value to use; 
     * the VisiPlot Options value or the constraint 
     * from the database.
     *
     * @return Moon Distance Constraint 
     * @see #readVisiPlotOptions(ObservationBlock ob, VisiPlotOptions vpo)
     * @see VisiPlotOptions#isReadFromOB(String value)
     */
    private double getMoonConstraint(
        ObservationBlock ob,
        VisiPlotOptions vpo,
        String value) {
        cs_ = ob.getConstraintSet();
        boolean readFromOB = VisiPlotOptions.isReadFromOB(value);
        if (!readFromOB) {
            try {
                moonConstraint_ = Double.valueOf(value).doubleValue();
            } catch (NumberFormatException e) {
                moonConstraint_ = 0.;
            }
        } else {
            moonConstraint_ = cs_.getMoonAngularDistance();
        }
        //stdlog_.info(" visiplot.Visiplot.getMoonConstraint " + moonConstraint_ + " " + value);
        return (moonConstraint_);
    }

}
